from datetime import datetime, timezone
from calendar import timegm

import jwt


class JWTCredentialsManager:
    """JWT for manage auth credentials"""

    def __init__(self, secret, expire_time):
        """expire_time is a timedelta
        secret is a string with the secret encoding key"""
        self.secret = secret
        self.expire_time = expire_time

    def get_token_for(self, user):
        """Generates a token with the brl_user and additional data dict if needed"""
        profile_fields = {"user": user,
                          "exp": timegm((datetime.now(timezone.utc) + self.expire_time).timetuple())}
        return jwt.encode(profile_fields, self.secret, algorithm="HS256")

    def get_user(self, token):
        """Gets the user from credentials object. None if no credentials.
        Can raise jwt.ExpiredSignature and jwt.DecodeError"""
        profile = jwt.decode(token, self.secret, algorithms=["HS256"])
        return profile.get("user", None)



from unittest.mock import patch

import pytest

from conan.tools.build import cmd_args_to_string
from conan.tools.gnu import AutotoolsToolchain
from conan.errors import ConanException
from conan.internal.model.conf import Conf
from conan.test.utils.mocks import ConanFileMock, MockSettings


@pytest.fixture()
def cross_building_conanfile():
    settings_build = MockSettings({"os": "Linux",
                                   "arch": "x86_64",
                                   "compiler": "gcc",
                                   "compiler.version": "11",
                                   "compiler.libcxx": "libstdc++",
                                   "build_type": "Release"})
    settings_target = MockSettings({"os": "Android", "arch": "armv8"})
    settings = MockSettings({"os": "Emscripten", "arch": "wasm"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.settings_build = settings_build
    conanfile.settings_target = settings_target
    return conanfile


def test_get_gnu_triplet_for_cross_building():
    """
    Testing AutotoolsToolchain and _get_gnu_triplet() function in case of
    having os=Windows and cross compiling
    """
    # Issue: https://github.com/conan-io/conan/issues/10139
    settings = MockSettings({"build_type": "Release",
                             "compiler": "gcc",
                             "compiler.version": "10.2",
                             "os": "Windows",
                             "arch": "x86_64"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.settings_build = MockSettings({"os": "Solaris", "arch": "x86"})
    autotoolschain = AutotoolsToolchain(conanfile)
    assert autotoolschain._host == "x86_64-w64-mingw32"
    assert autotoolschain._build == "i686-solaris"


def test_get_toolchain_cppstd():
    settings = MockSettings({"build_type": "Release",
                             "compiler": "gcc",
                             "compiler.version": "9",
                             "compiler.cppstd": "20",
                             "os": "Linux",
                             "arch": "x86_64"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.settings_build = settings
    autotoolschain = AutotoolsToolchain(conanfile)
    assert autotoolschain.cppstd == "-std=c++2a"
    settings.values["compiler.version"] = "10"
    autotoolschain = AutotoolsToolchain(conanfile)
    assert autotoolschain.cppstd == "-std=c++20"


@pytest.mark.parametrize("runtime, runtime_type, expected",
                         [("static", "Debug", "MTd"),
                          ("static", "Release", "MT"),
                          ("dynamic", "Debug", "MDd"),
                          ("dynamic", "Release", "MD")])
def test_msvc_runtime(runtime, runtime_type, expected):
    """
    Testing AutotoolsToolchain with the msvc compiler adjust the runtime
    """
    # Issue: https://github.com/conan-io/conan/issues/10139
    settings = MockSettings({"build_type": "Release",
                             "compiler": "msvc",
                             "compiler.runtime": runtime,
                             "compiler.runtime_type": runtime_type,
                             "os": "Windows",
                             "arch": "x86_64"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.settings_build = settings
    autotoolschain = AutotoolsToolchain(conanfile)
    expected_flag = "-{}".format(expected)
    assert autotoolschain.msvc_runtime_flag == expected_flag
    env = autotoolschain.environment().vars(conanfile)
    assert expected_flag in env["CFLAGS"]
    assert expected_flag in env["CXXFLAGS"]


@pytest.mark.parametrize("runtime", ["MTd", "MT", "MDd", "MD"])
def test_visual_runtime(runtime):
    """
    Testing AutotoolsToolchain with the msvc compiler adjust the runtime
    """
    # Issue: https://github.com/conan-io/conan/issues/10139
    settings = MockSettings({"build_type": "Release" if "d" not in runtime else "Debug",
                             "compiler": "msvc",
                             "compiler.runtime": "static" if "MT" in runtime else "dynamic",
                             "compiler.runtime_type": "Release" if "d" not in runtime else "Debug",
                             "os": "Windows",
                             "arch": "x86_64"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.settings_build = settings
    autotoolschain = AutotoolsToolchain(conanfile)
    expected_flag = "-{}".format(runtime)
    assert autotoolschain.msvc_runtime_flag == expected_flag
    env = autotoolschain.environment().vars(conanfile)
    assert expected_flag in env["CFLAGS"]
    assert expected_flag in env["CXXFLAGS"]


def test_get_gnu_triplet_for_cross_building_raise_error():
    """
    Testing AutotoolsToolchain and _get_gnu_triplet() function raises an error in case of
    having os=Windows, cross compiling and not defined any compiler
    """
    # Issue: https://github.com/conan-io/conan/issues/10139
    settings = MockSettings({"build_type": "Release",
                             "os": "Windows",
                             "arch": "x86_64"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.settings_build = MockSettings({"os": "Solaris", "arch": "x86"})
    with pytest.raises(ConanException) as conan_error:
        AutotoolsToolchain(conanfile)
    msg = "'compiler' parameter for 'get_gnu_triplet()' is not specified and needed for os=Windows"
    assert msg == str(conan_error.value)


def test_compilers_mapping():
    autotools_mapping = {"c": "CC", "cpp": "CXX", "cuda": "NVCC", "fortran": "FC"}
    compilers = {"c": "path_to_c", "cpp": "path_to_cpp", "cuda": "path_to_cuda",
                 "fortran": "path_to_fortran"}
    settings = MockSettings({"build_type": "Release",
                             "os": "Windows",
                             "arch": "x86_64",
                             "compiler": "gcc"})
    conanfile = ConanFileMock()
    conanfile.conf = Conf()
    conanfile.conf.define("tools.build:compiler_executables", compilers)
    conanfile.settings = settings
    autotoolschain = AutotoolsToolchain(conanfile)
    env = autotoolschain.environment().vars(conanfile)
    for compiler, env_var in autotools_mapping.items():
        assert env[env_var] == f"path_to_{compiler}"


def test_linker_scripts():
    conanfile = ConanFileMock()
    conanfile.conf = Conf()
    conanfile.conf.define("tools.build:linker_scripts", ["path_to_first_linker_script", "path_to_second_linker_script"])
    settings = MockSettings({"build_type": "Release",
                             "os": "Windows",
                             "compiler": "gcc",
                             "arch": "x86_64"})
    conanfile.settings = settings
    autotoolschain = AutotoolsToolchain(conanfile)
    env = autotoolschain.environment().vars(conanfile)
    assert "-T'path_to_first_linker_script'" in env["LDFLAGS"]
    assert "-T'path_to_second_linker_script'" in env["LDFLAGS"]


@patch("conan.tools.gnu.autotoolstoolchain.save_toolchain_args")
def test_check_configure_args_overwriting_and_deletion(save_args, cross_building_conanfile):
    # Issue: https://github.com/conan-io/conan/issues/12642
    at = AutotoolsToolchain(cross_building_conanfile)
    at.configure_args.extend([
        "--with-cross-build=my_path",
        "--something-host=my_host"
    ])
    at.generate_args()
    configure_args = save_args.call_args[0][0]['configure_args']
    assert "--build=x86_64-linux-gnu" in configure_args
    assert "--host=wasm32-local-emscripten" in configure_args
    assert "--with-cross-build=my_path" in configure_args
    assert "--something-host=my_host" in configure_args
    # https://github.com/conan-io/conan/issues/12431
    at.configure_args.remove("--build=x86_64-linux-gnu")
    at.configure_args.remove("--host=wasm32-local-emscripten")
    at.generate_args()
    configure_args = save_args.call_args[0][0]['configure_args']
    assert "--build=x86_64-linux-gnu" not in configure_args  # removed
    assert "--host=wasm32-local-emscripten" not in configure_args  # removed
    assert "--with-cross-build=my_path" in configure_args
    assert "--something-host=my_host" in configure_args


def test_update_or_prune_any_args(cross_building_conanfile):
    at = AutotoolsToolchain(cross_building_conanfile)
    at.configure_args.append("--enable-flag1=false")
    # Update configure_args
    at.update_configure_args({"--prefix": "/my/other/prefix",
                              "--build": None,  # prune value
                              "--enable-flag1": "",  # without value
                              "-NEW-FLAG": "no"  # new flag
                              })
    new_configure_args = cmd_args_to_string(at.configure_args)
    assert "--prefix=/my/other/prefix" in new_configure_args
    assert "--build=" not in new_configure_args  # pruned
    assert "--enable-flag1" in new_configure_args  # flag without value
    assert "-NEW-FLAG=no" in new_configure_args  # new flag
    # Update autoreconf_args
    at.update_autoreconf_args({"--force": None})
    new_autoreconf_args = cmd_args_to_string(at.autoreconf_args)
    assert "'--force" not in new_autoreconf_args
    # Add new value to make_args
    at.update_make_args({"--new-complex-flag": "new-value"})
    new_make_args = cmd_args_to_string(at.make_args)
    assert "--new-complex-flag=new-value" in new_make_args


def test_tricore():
    settings = MockSettings({"build_type": "Release",
                             "compiler": "gcc",
                             "os": "baremetal",
                             "arch": "tc131"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.settings_build = settings
    autotoolschain = AutotoolsToolchain(conanfile)
    env = autotoolschain.environment().vars(conanfile)
    assert '-mtc131' in env["CFLAGS"]
    assert '-mtc131' in env["CXXFLAGS"]
    assert '-mtc131' in env["LDFLAGS"]



from unittest.mock import patch

from conan import __version__
from conan.test.utils.env import environment_update
from conan.test.utils.tools import TestClient


class TestHelp:

    def test_version(self):
        c = TestClient()
        c.run("--version")
        assert "Conan version %s" % __version__ in c.out

    def test_unknown_command(self):
        c = TestClient()
        c.run("some_unknown_command123", assert_error=True)
        assert "'some_unknown_command123' is not a Conan command" in c.out

    def test_similar(self):
        c = TestClient()
        c.run("instal", assert_error=True)
        assert "The most similar command is" in c.out
        assert "install" in c.out

        c.run("remole", assert_error=True)
        assert "The most similar commands are" in c.out
        assert "remove" in c.out
        assert "remote" in c.out

    def test_help(self):
        c = TestClient()
        c.run("-h")
        assert "Creator commands" in c.out
        assert "Consumer commands" in c.out

    def test_help_command(self):
        c = TestClient()
        c.run("new -h")
        assert "Create a new example recipe" in c.out

    def test_help_subcommand(self):
        c = TestClient()
        c.run("cache -h")
        # When listing subcommands, but line truncated
        assert "Perform file operations in the local cache (of recipes and/or packages)" in c.out
        c.run("cache path -h")
        assert "Show the path to the Conan cache for a given reference" in c.out


def test_all_commands_call_args_parse():
    tc = TestClient(light=True)
    tc.run("-h")
    commands = tc.api.command.cli._builtin_commands
    with environment_update({"CONAN_WORKSPACE_ENABLE": "will_break_next"}):
        for command, info in commands.items():
            if len(info._subcommands) > 0:
                for subcommand in info._subcommands.values():
                    with patch("conan.cli.command.ConanArgumentParser.parse_args",
                               side_effect=Exception("called")) as mock_run:
                        try:
                            tc.run(f"{command} {subcommand.name} -h")
                        except:
                            pass
                        finally:
                            assert mock_run.called, f'Command "conan {command} {subcommand.name}" did not call parse_args()'
            else:
                with patch("conan.cli.command.ConanArgumentParser.parse_args",
                           side_effect=Exception("called")) as mock_run:
                    try:
                        tc.run(f"{command} -h")
                    except:
                        pass
                    finally:
                        assert mock_run.called, f'Command "conan {command}" did not call parse_args()'



from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


class TestEditablePackageTest:

    def test_install_ok(self):
        ref = "--name=lib --version=version  --user=user --channel=name"
        t = TestClient()
        t.save({'conanfile.py': GenConanfile()})
        t.run('editable add {}'.format(ref))
        assert "Reference 'lib/version@user/name' in editable mode" in t.out

    def test_editable_list_search(self):
        ref = "--name=lib --version=version  --user=user --channel=name"
        t = TestClient()
        t.save({'conanfile.py': GenConanfile()})
        t.run('editable add . {}'.format(ref))
        t.run("editable list")
        assert "lib/version@user/name" in t.out
        assert "    Path:" in t.out

    def test_missing_subarguments(self):
        t = TestClient()
        t.run("editable", assert_error=True)
        assert "ERROR: Exiting with code: 2" in t.out

    def test_conanfile_name(self):
        t = TestClient()
        t.save({'othername.py': GenConanfile("lib", "version")})
        t.run('editable add ./othername.py --user=user --channel=name')
        assert "Reference 'lib/version@user/name' in editable mode" in t.out
        t.run('install --requires=lib/version@user/name')
        t.assert_listed_require({"lib/version@user/name": "Editable"})

    def test_pyrequires_remote(self):
        t = TestClient(default_server_user=True)
        t.save({"conanfile.py": GenConanfile("pyreq", "1.0")})
        t.run("create .")
        t.run("upload pyreq/1.0 -c -r=default")
        t.run("remove pyreq/1.0 -c")
        t.save({"conanfile.py": GenConanfile("pkg", "1.0").with_python_requires("pyreq/1.0")})
        t.run("editable add . -nr", assert_error=True)
        assert "Cannot resolve python_requires 'pyreq/1.0': No remote defined" in t.out
        t.run("editable add .")
        assert "Reference 'pkg/1.0' in editable mode" in t.out


def test_editable_no_name_version_test_package():
    tc = TestClient()
    tc.save({"conanfile.py": GenConanfile(),
             "test_package/conanfile.py": GenConanfile("test_package")
             .with_class_attribute("test_type = 'explicit'")
            .with_test("self.output.info('Testing the package')")})
    tc.run("editable add . --name=foo", assert_error=True)
    assert "ERROR: Editable package recipe should declare its name and version" in tc.out

    tc.run("editable add . --version=1.0", assert_error=True)
    assert "ERROR: Editable package recipe should declare its name and version" in tc.out

    tc.run("editable add .", assert_error=True)
    assert "ERROR: Editable package recipe should declare its name and version" in tc.out



import os

from conan.api.model import RecipeReference
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.test_files import uncompress_packaged_files
from conan.test.utils.tools import TestClient


def test_reuse_uploaded_tgz():
    client = TestClient(default_server_user=True)
    # Download packages from a remote, then copy to another channel
    # and reupload them. Because they have not changed, the tgz is not created again

    # UPLOAD A PACKAGE
    ref = RecipeReference.loads("hello0/0.1@user/stable")
    files = {"conanfile.py": GenConanfile("hello0", "0.1").with_exports("*"),
             "another_export_file.lib": "to compress"}
    client.save(files)
    client.run("create . --user=user --channel=stable")
    client.run("upload %s -r default" % str(ref))
    assert "Compressing conan_export.tgz" in client.out
    assert "Compressing conan_package.tgz" in client.out


def test_reuse_downloaded_tgz():
    # Download packages from a remote, then copy to another channel
    # and reupload them. It needs to compress it again, not tgz is kept
    client = TestClient(default_server_user=True)
    # UPLOAD A PACKAGE
    files = {"conanfile.py": GenConanfile("hello0", "0.1").with_exports("*"),
             "another_export_file.lib": "to compress"}
    client.save(files)
    client.run("create . --user=user --channel=stable")
    client.run("upload hello0/0.1@user/stable -r default")
    assert "Compressing conan_export.tgz" in client.out
    assert "Compressing conan_package.tgz" in client.out

    # Other user downloads the package
    # THEN A NEW USER DOWNLOADS THE PACKAGES AND UPLOADS COMPRESSING AGAIN
    # BECAUSE ONLY TGZ IS KEPT WHEN UPLOADING
    other_client = TestClient(servers=client.servers, inputs=["admin", "password"])
    other_client.run("download hello0/0.1@user/stable -r default")
    other_client.run("upload hello0/0.1@user/stable -r default")
    assert "Compressing conan_export.tgz" in client.out
    assert "Compressing conan_package.tgz" in client.out


def test_upload_only_tgz_if_needed():
    client = TestClient(default_server_user=True)
    ref = RecipeReference.loads("hello0/0.1@user/stable")
    conanfile = GenConanfile("hello0", "0.1").with_exports("*").with_package_file("lib/file.lib",
                                                                                  "File")
    client.save({"conanfile.py": conanfile,
                 "file.txt": "contents"})
    client.run("create . --user=user --channel=stable")

    # Upload conans
    client.run("upload %s -r default --only-recipe" % str(ref))
    assert "Compressing conan_export.tgz" in client.out

    # Not needed to tgz again
    client.run("upload %s -r default --only-recipe" % str(ref))
    assert "Compressing" not in client.out

    # Check that conans exists on server
    server_paths = client.servers["default"].server_store
    conan_path = server_paths.conan_revisions_root(ref)
    assert os.path.exists(conan_path)

    latest_rrev = client.cache.get_latest_recipe_revision(ref)
    package_ids = client.cache.get_package_references(latest_rrev)
    pref = package_ids[0]

    # Upload package
    client.run("upload %s#*:%s -r default -c" % (str(ref), str(pref.package_id)))
    assert "Compressing conan_package.tgz" in client.out

    # Not needed to tgz again
    client.run("upload %s#*:%s -r default -c" % (str(ref), str(pref.package_id)))
    assert "Compressing" not in client.out

    # If we install the package again will be removed and re tgz
    client.run("install --requires=%s --build missing" % str(ref))
    # Upload package
    client.run("upload %s#*:%s -r default -c" % (str(ref), str(pref.package_id)))
    assert "Compressing" not in client.out

    # Check library on server
    folder = uncompress_packaged_files(server_paths, pref)
    libraries = os.listdir(os.path.join(folder, "lib"))
    assert len(libraries) == 1



import os
import platform

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient, NO_SETTINGS_PACKAGE_ID


@pytest.mark.skipif(platform.system() == "Windows", reason="Uses symlinks")
def test_custom_symlinked_home():
    base_cache = temp_folder()
    real_cache = os.path.join(base_cache, "real_cache")
    os.makedirs(real_cache)
    symlink_cache = os.path.join(base_cache, "symlink_cache")
    os.symlink(real_cache, symlink_cache)
    c = TestClient(cache_folder=symlink_cache)
    c.save({"conanfile.py": GenConanfile("pkg", "0.1")})
    c.run("create .")
    assert "symlink_cache" in c.out
    assert "real_cache" not in c.out
    c.run("cache path pkg/0.1")
    assert "symlink_cache" in c.out
    assert "real_cache" not in c.out
    c.run(f"cache path pkg/0.1:{NO_SETTINGS_PACKAGE_ID}")
    assert "symlink_cache" in c.out
    assert "real_cache" not in c.out



import os
import stat
import pytest

from conan.test.utils.test_files import temp_folder
from conan.internal.util.files import remove, save


class TestRemove:

    @pytest.fixture
    def setup_file(self):
        file = os.path.join(temp_folder(), 'file.txt')
        save(file, "some content")
        return file

    def test_remove(self, setup_file):
        remove(setup_file)
        assert not os.path.exists(setup_file)
        assert os.path.exists(os.path.dirname(setup_file))

    def test_remove_readonly(self, setup_file):
        os.chmod(setup_file, stat.S_IREAD|stat.S_IRGRP|stat.S_IROTH)
        with pytest.raises((IOError, OSError)):
            save(setup_file, "change the content")
        remove(setup_file)
        assert not os.path.exists(setup_file)
        assert os.path.exists(os.path.dirname(setup_file))

    def test_remove_folder(self, setup_file):
        dirname = os.path.dirname(setup_file)
        with pytest.raises(AssertionError):
            remove(dirname)
        assert os.path.exists(dirname)



import os

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


class TestRemoveEmptyDirs:

    def test_basic(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("hello", "0.1")})
        client.run("export . --user=lasote --channel=stable")
        ref_layout = client.exported_layout()
        assert os.path.exists(ref_layout.base_folder)
        client.run("remove hello* -c")
        assert not os.path.exists(ref_layout.base_folder)

    def test_shared_folder(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("hello", "0.1")})
        client.run("export . --user=lasote --channel=stable")
        ref_layout = client.exported_layout()
        assert os.path.exists(ref_layout.base_folder)
        client.run("export . --user=lasote2 --channel=stable")
        ref_layout2 = client.exported_layout()
        assert os.path.exists(ref_layout2.base_folder)
        client.run("remove hello/0.1@lasote/stable -c")
        assert not os.path.exists(ref_layout.base_folder)
        assert os.path.exists(ref_layout2.base_folder)



import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_build_requires_ranges():
    # app -> pkga ----------> pkgb ------------> pkgc
    #          \-cmake/[*]     \ -cmake/1.0        \-cmake/[*]
    # The resolution of cmake/[*] is invariant, it will always resolved to cmake/0.5, not
    # one to cmake/0.5 and the next one to cmake/1.0 because in between there was an explicit
    # dependency to cmake/1.0
    client = TestClient(default_server_user=True)
    client.save({"conanfile.py": GenConanfile()})

    client.run("create . --name=cmake --version=0.5")
    client.run("create . --name=cmake --version=1.0")
    client.run("upload cmake/1.0* -c -r default")
    client.run("remove cmake/1.0* -c")

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            {}
            tool_requires = "cmake/{}"
            def generate(self):
                for r, d in self.dependencies.items():
                    self.output.info("REQUIRE {{}}: {{}}".format(r.ref, d))
                dep = self.dependencies.build.get("cmake")
                self.output.info("CMAKEVER: {{}}!!".format(dep.ref.version))
            """)
    client.save({"pkgc/conanfile.py": conanfile.format("", "[*]"),
                 "pkgb/conanfile.py": conanfile.format("requires = 'pkgc/1.0'", "1.0"),
                 "pkga/conanfile.py": conanfile.format("requires = 'pkgb/1.0'", "[*]"),
                 })
    client.run("export pkgc --name=pkgc --version=1.0")
    client.run("export pkgb --name=pkgb --version=1.0")
    client.run("export pkga --name=pkga --version=1.0")

    client.run("install --requires=pkga/1.0@ --build=missing")
    assert "pkgc/1.0: REQUIRE cmake/0.5: cmake/0.5" in client.out
    assert "pkgc/1.0: CMAKEVER: 0.5!!" in client.out
    assert "pkgb/1.0: REQUIRE cmake/1.0: cmake/1.0" in client.out
    assert "pkgb/1.0: CMAKEVER: 1.0!!" in client.out
    assert "pkga/1.0: REQUIRE cmake/0.5: cmake/0.5" in client.out
    assert "pkga/1.0: CMAKEVER: 0.5!!" in client.out



from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_info_options():
    # packages with dash
    client = TestClient()
    client.save({"conanfile.py": GenConanfile("my-package", "1.3")
                .with_option("shared", [True, False])
                .with_default_option("shared", False)})

    # local
    client.run("graph info .")
    assert "shared: False" in client.out
    client.run("graph info . -o shared=True")
    assert "shared: True" in client.out
    client.run("graph info . -o my-package*:shared=True")
    assert "shared: True" in client.out

    # in cache
    client.run("export .")
    client.run("graph info --requires=my-package/1.3")
    assert "shared: False" in client.out
    client.run("graph info --requires=my-package/1.3 -o shared=True")
    assert "shared: True" in client.out
    client.run("graph info --requires=my-package/1.3 -o my-package*:shared=True")
    assert "shared: True" in client.out




import textwrap

from conan.api.model import RecipeReference
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.internal.util.files import load, save


def test_legacy_names_filenames():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "pkg"
            version = "1.0"
            def package_info(self):
                self.cpp_info.components["comp"].names["cmake_find_package"] = "hello"
                self.cpp_info.components["comp"].names["cmake_find_package_multi"] = "hello"
                self.cpp_info.components["comp"].build_modules["cmake_find_package"] = ["nice_rel_path"]
                self.cpp_info.components["comp"].build_modules["cmake_find_package"].append("some_file_name")
                self.cpp_info.components["comp"].build_modules["cmake_find_package_multi"] = ["nice_rel_path"]

                self.cpp_info.names["cmake_find_package"] = "absl"
                self.cpp_info.names["cmake_find_package_multi"] = "absl"
                self.cpp_info.filenames["cmake_find_package"] = "tensorflowlite"
                self.cpp_info.filenames["cmake_find_package_multi"] = "tensorflowlite"
                self.cpp_info.build_modules["cmake_find_package"] = ["nice_rel_path"]
                self.cpp_info.build_modules["cmake_find_package_multi"] = ["nice_rel_path"]

                self.env_info.whatever = "whatever-env_info"
                self.env_info.PATH.append("/path/to/folder")
                self.user_info.whatever = "whatever-user_info"
        """)
    c.save({"conanfile.py": conanfile})
    c.run("create .")
    for name in ["cpp_info.names", "cpp_info.filenames", "env_info", "user_info", "cpp_info.build_modules"]:
        assert f"WARN: deprecated:     '{name}' used in: pkg/1.0" in c.out

    c.save_home({"global.conf": 'core:skip_warnings=["deprecated"]'})
    c.run("create .")
    for name in ["cpp_info.names", "cpp_info.filenames", "env_info", "user_info",
                 "cpp_info.build_modules"]:
        assert f"'{name}' used in: pkg/1.0" not in c.out


class TestLegacy1XRecipes:
    def test_legacy_imports(self):
        c = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                name = "pkg"
                version = "1.0"
            """)
        c.save({"pkg/conanfile.py": conanfile,
                "app/conanfile.py": GenConanfile("app", "1.0").with_requires("pkg/1.0")})
        # With EDITABLE, we can emulate errors without exporting
        c.run("export pkg")
        layout = c.get_latest_ref_layout(RecipeReference.loads("pkg/1.0"))
        conanfile = layout.conanfile()
        content = load(conanfile)
        content = content.replace("from conan", "from conans")
        save(conanfile, content)
        c.run("install app", assert_error=True)
        assert "Recipe 'pkg/1.0' seems broken." in c.out
        assert "It is possible that this recipe is not Conan 2.0 ready" in c.out




import textwrap

import pytest

from conan.api.model import RecipeReference
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_dependencies_visit():
    client = TestClient(light=True)
    client.save({"conanfile.py": GenConanfile().with_shared_option(True)})
    client.run("create . --name=openssl --version=0.1")
    client.run("create . --name=openssl --version=0.2")
    client.save({"conanfile.py": GenConanfile().with_requires("openssl/0.2")})
    client.run("create . --name=cmake --version=0.1")
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            requires = "openssl/0.1"
            build_requires = "cmake/0.1"

            def generate(self):
                dep = self.dependencies["openssl"]
                self.output.info("DefRef: {}!!!".format(dep.ref.repr_notime()))
                self.output.info("DefPRef: {}!!!".format(dep.pref.repr_notime()))
                dep = self.dependencies.build["openssl"]
                self.output.info("DefRefBuild: {}!!!".format(dep.ref.repr_notime()))
                self.output.info("DefPRefBuild: {}!!!".format(dep.pref.repr_notime()))
                for r, d in self.dependencies.build.items():
                    self.output.info("DIRECTBUILD {}: {}".format(r.direct, d))

                if "openssl" in self.dependencies:
                    self.output.info("OpenSSL found in deps")

                if "cmake" in self.dependencies:
                    self.output.info("cmake found in default deps")

                if "cmake" in self.dependencies.build:
                    self.output.info("cmake found in deps.build")

                if "badlib" in self.dependencies:
                    self.output.info("badlib found in deps")
        """)
    client.save({"conanfile.py": conanfile})

    client.run("install .")
    refs = client.cache.get_latest_recipe_revision(RecipeReference.loads("openssl/0.1"))
    pkgs = client.cache.get_package_references(refs)
    prev1 = client.cache.get_latest_package_revision(pkgs[0])
    assert f"DefRef: {repr(prev1.ref)}!!!" in client.out
    assert f"DefPRef: {prev1.repr_notime()}!!!" in client.out

    refs = client.cache.get_latest_recipe_revision(RecipeReference.loads("openssl/0.2"))
    pkgs = client.cache.get_package_references(refs)
    prev2 = client.cache.get_latest_package_revision(pkgs[0])
    assert f"DefRefBuild: {repr(prev2.ref)}!!!" in client.out
    assert f"DefPRefBuild: {prev2.repr_notime()}!!!" in client.out

    assert "conanfile.py: DIRECTBUILD True: cmake/0.1" in client.out
    assert "conanfile.py: DIRECTBUILD False: openssl/0.2" in client.out

    assert "OpenSSL found in deps" in client.out
    assert "badlib found in deps" not in client.out

    assert "cmake found in default deps" not in client.out
    assert "cmake found in deps.build" in client.out


def test_dependencies_visit_settings_options():
    client = TestClient(light=True)
    client.save({"conanfile.py": GenConanfile().with_settings("os").
                with_option("shared", [True, False]).with_default_option("shared", False)})
    client.run("create . --name=openssl --version=0.1 -s os=Linux")
    client.save({"conanfile.py": GenConanfile().with_requires("openssl/0.1")})
    client.run("create . --name=pkg --version=0.1  -s os=Linux")
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            requires = "pkg/0.1"

            def generate(self):
                dep = self.dependencies["openssl"]
                self.output.info("SETTINGS: {}!".format(dep.settings.os))
                self.output.info("OPTIONS: shared={}!".format(dep.options.shared))
        """)
    client.save({"conanfile.py": conanfile})
    client.run("install . -s os=Linux")
    assert "conanfile.py: SETTINGS: Linux!" in client.out
    assert "conanfile.py: OPTIONS: shared=False!" in client.out


asserts = [
    ('print("=>{}".format(self.dependencies["zlib"].ref))',
     False, "=>zlib/0.2"),
    ('print("=>{}".format(self.dependencies.build.get("zlib").ref))',
     False, "=>zlib/0.1"),
    ('print("=>{}".format(self.dependencies.get("zlib", build=True).ref))',
     False, "=>zlib/0.1"),
    ('print("=>{}".format(self.dependencies.get("zlib", build=False).ref))',
     False, "=>zlib/0.2"),
    ('print("=>{}".format(self.dependencies.get("zlib", build=True, visible=False).ref))',
     False, "=>zlib/0.1"),
    ('self.dependencies.get("cmake", build=True)', True,
     'There are more than one requires matching the specified filters: {\'build\': True}\n'
     '- cmake/0.1, Traits: build=True, headers=False, libs=False, run=False, visible=False\n'
     '- cmake/0.2, Traits: build=True, headers=False, libs=False, run=False, visible=False'
     ),
    ('self.dependencies["missing"]', True, "'missing' not found in the dependency set"),
    ('self.output.info("Missing in deps: " + str("missing" in self.dependencies))', False, "Missing in deps: False"),
    ('self.output.info("Zlib in deps: " + str("zlib" in self.dependencies))', False, "Zlib in deps: True"),
    ('self.output.info("Zlib in deps.build: " + str("zlib" in self.dependencies.build))', False, "Zlib in deps.build: True"),
]


@pytest.mark.parametrize("generates_line, assert_error, output_text", asserts)
def test_cmake_zlib(generates_line, assert_error, output_text):
    # app (br)---> cmake (0.1) -> zlib/0.1
    #  \     ----> zlib/0.2
    #  \  (br)---> cmake (0.2)
    client = TestClient(light=True)
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=zlib --version=0.1")
    client.run("create . --name=zlib --version=0.2")

    client.save({"conanfile.py": GenConanfile().with_tool_requirement("zlib/0.1",
                                                                      visible=True)})
    client.run("create . --name=cmake --version=0.1")

    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=cmake --version=0.2")

    app_conanfile = textwrap.dedent("""
    from conan import ConanFile
    class Pkg(ConanFile):

        def requirements(self):
            self.requires("zlib/0.2")
            self.requires("cmake/0.1", headers=False, libs=False, visible=False, build=True, run=False)
            self.requires("cmake/0.2", headers=False, libs=False, visible=False, build=True, run=False)

        def generate(self):
           {}
        """.format(generates_line))
    client.save({"conanfile.py": app_conanfile})
    client.run("create . --name=app --version=1.0", assert_error=assert_error)
    assert output_text in client.out


def test_invisible_not_colliding_test_requires():
    """
    The test_requires are private, so the app can't see them, only the foo and bar
    :return:
    """
    # app ---> foo/0.1 --test_require--> gtest/0.1
    #  \  ---> bar/0.1 --test_require--> gtest/0.2
    client = TestClient(light=True)
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=gtest --version=0.1")
    client.run("create . --name=gtest --version=0.2")

    client.save({"conanfile.py": GenConanfile().with_test_requires("gtest/0.1")})
    client.run("create . --name=foo --version=0.1")

    client.save({"conanfile.py": GenConanfile().with_test_requires("gtest/0.2")})
    client.run("create . --name=bar --version=0.1")

    app_conanfile = textwrap.dedent("""
    from conan import ConanFile
    class Pkg(ConanFile):

        def requirements(self):
            self.requires("foo/0.1")
            self.requires("bar/0.1")

        def generate(self):
            print(self.dependencies.get("gtest"))
        """)
    client.save({"conanfile.py": app_conanfile})
    client.run("create . --name=app --version=1.0", assert_error=True)
    assert "'gtest' not found in the dependency set" in client.out


def test_dependencies_visit_build_requires_profile():
    """ At validate() time, in Conan 1.X, the build-requires are not available yet, because first
    the binary package_id is computed, then the build-requires are resolved.
    It is necessary to avoid in Conan 1.X the caching of the ConanFile.dependencies, because
    at generate() time it will have all the graph info, including build-requires
    """
    # https://github.com/conan-io/conan/issues/10304
    client = TestClient(light=True)
    client.save({"conanfile.py": GenConanfile("cmake", "0.1")})
    client.run("create .")
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):

            def validate(self):
                self.output.info("VALIDATE DEPS: {}!!!".format(len(self.dependencies.items())))

            def generate(self):
                for req, dep in self.dependencies.items():
                    self.output.info("GENERATE REQUIRE: {}!!!".format(dep.ref))
                dep = self.dependencies.build["cmake"]
                self.output.info("GENERATE CMAKE: {}!!!".format(dep.ref))
        """)
    client.save({"conanfile.py": conanfile,
                 "profile": "[tool_requires]\ncmake/0.1"})
    client.run("install . -pr:b=default -pr:h=profile --build='*'")  # Use 2 contexts

    # Validate time, build-requires available
    assert "conanfile.py: VALIDATE DEPS: 1!!!" in client.out
    # generate time, build-requires already available
    assert "conanfile.py: GENERATE REQUIRE: cmake/0.1!!!" in client.out
    assert "conanfile.py: GENERATE CMAKE: cmake/0.1!!!" in client.out


def test_dependencies_package_type():
    c = TestClient(light=True)
    c.save({"conanfile.py": GenConanfile("lib", "0.1").with_package_type("static-library")})
    c.run("create .")
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            requires = "lib/0.1"
            def generate(self):
                is_app = self.dependencies["lib"].package_type == "static-library"
                self.output.info(f"APP: {is_app}!!")
                assert is_app
                self.dependencies["lib"].package_type == "not-exist-type"
        """)
    c.save({"conanfile.py": conanfile})
    c.run("install .", assert_error=True)
    assert "APP: True!!" in c.out
    assert "conanfile.py: Error in generate() method, line 9" in c.out
    assert "ValueError: 'not-exist-type' is not a valid PackageType" in c.out


def test_dependency_interface():
    """
    Quick test for the ConanFileInterface exposed
    """
    c = TestClient(light=True)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "dep"
            version = "1.0"
            homepage = "myhome"
            url = "myurl"
            license = "MIT"
        """)
    user = textwrap.dedent("""
        from conan import ConanFile
        class User(ConanFile):
            requires = "dep/1.0"
            def generate(self):
                dep = self.dependencies["dep"]
                self.output.info("HOME: {}".format(dep.homepage))
                self.output.info("URL: {}".format(dep.url))
                self.output.info("LICENSE: {}".format(dep.license))
                self.output.info("RECIPE FOLDER: {}".format(dep.recipe_folder))
                self.output.info("CONANDATA: {}".format(dep.conan_data))
                self.output.info("RECIPE: {}".format(dep.recipe))

            """)
    c.save({"dep/conanfile.py": conanfile,
            "dep/conandata.yml": "",
            "user/conanfile.py": user})
    c.run("create dep")
    c.run("install user")
    assert "conanfile.py: HOME: myhome" in c.out
    assert "conanfile.py: URL: myurl" in c.out
    assert "conanfile.py: LICENSE: MIT" in c.out
    assert "conanfile.py: RECIPE FOLDER:" in c.out
    assert "conanfile.py: CONANDATA: {}" in c.out
    assert "conanfile.py: RECIPE: Cache" in c.out


def test_dependency_interface_validate():
    """
    In the validate() method, there is no access to dep.package_folder, because the packages are
    not there. validate() operates at the graph level, without binaries, before they are installed,
    and we want to keep it that way
    https://github.com/conan-io/conan/issues/11959
    """
    c = TestClient(light=True)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "dep"
            version = "1.0"
            homepage = "myhome"
        """)
    user = textwrap.dedent("""
        import os
        from conan import ConanFile
        class User(ConanFile):
            requires = "dep/1.0"
            def validate(self):
                dep = self.dependencies["dep"]
                self.output.info("HOME: {}".format(dep.homepage))
                self.output.info("PKG FOLDER: {}".format(dep.package_folder is None))
            """)
    c.save({"dep/conanfile.py": conanfile,
            "user/conanfile.py": user})
    c.run("create dep")
    c.run("install user")
    assert "conanfile.py: HOME: myhome" in c.out
    assert "conanfile.py: PKG FOLDER: True" in c.out


def test_validate_visibility():
    # https://github.com/conan-io/conan/issues/12027
    c = TestClient(light=True)
    t1 = textwrap.dedent("""
        from conan import ConanFile
        class t1Conan(ConanFile):
            name = "t1"
            version = "0.1"
            package_type = "static-library"

            def package_info(self):
                self.cpp_info.libs = ["mylib"]
        """)
    t2 = textwrap.dedent("""
        from conan import ConanFile
        class t2Conan(ConanFile):
            name = "t2"
            version = "0.1"
            requires = "t1/0.1"
            package_type = "shared-library"

            def validate(self):
                self.output.info("VALID: {}".format(self.dependencies["t1"]))
        """)
    t3 = textwrap.dedent("""
        from conan import ConanFile
        class t3Conan(ConanFile):
            name = "t3"
            version = "0.1"
            requires = "t2/0.1"
            package_type = "application"

            def validate(self):
                self.output.info("VALID: {}".format(self.dependencies["t1"]))
                self.output.info("VALID: {}".format(self.dependencies["t2"]))

            def generate(self):
                self.output.info("GENERATE: {}".format(self.dependencies["t1"]))
                self.output.info("GENERATE: {}".format(self.dependencies["t2"]))
        """)

    c.save({"t1/conanfile.py": t1,
            "t2/conanfile.py": t2,
            "t3/conanfile.py": t3})
    c.run("create t1")
    c.run("create t2")
    c.run("install t3")
    assert "t2/0.1: VALID: t1/0.1" in c.out
    assert "conanfile.py (t3/0.1): VALID: t1/0.1" in c.out
    assert "conanfile.py (t3/0.1): VALID: t2/0.1" in c.out
    assert "conanfile.py (t3/0.1): GENERATE: t1/0.1" in c.out
    assert "conanfile.py (t3/0.1): GENERATE: t2/0.1" in c.out
    c.run("create t3")
    assert "t2/0.1: VALID: t1/0.1" in c.out
    assert "t3/0.1: VALID: t1/0.1" in c.out
    assert "t3/0.1: VALID: t2/0.1" in c.out
    assert "t3/0.1: GENERATE: t1/0.1" in c.out
    assert "t3/0.1: GENERATE: t2/0.1" in c.out



import os
import shutil
import textwrap

import pytest

from conan.test.assets.sources import gen_function_cpp, gen_function_h
from conan.test.utils.tools import TestClient


@pytest.mark.tool("cmake")
def test_editable_cmake_components():
    hello_h = gen_function_h(name="hello")
    hello_cpp = gen_function_cpp(name="hello", includes=["hello"])
    bye_h = gen_function_h(name="bye")
    bye_cpp = gen_function_cpp(name="bye", includes=["bye"])

    conanfile_greetings = textwrap.dedent("""
        from os.path import join
        from conan import ConanFile
        from conan.tools.cmake import CMake, cmake_layout
        from conan.tools.files import copy

        class GreetingsConan(ConanFile):
            name = "greetings"
            version = "0.1"
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeDeps", "CMakeToolchain"
            exports_sources = "src/*"

            def build(self):
               cmake = CMake(self)
               cmake.configure()
               cmake.build()

            def layout(self):
                cmake_layout(self, src_folder="src")
                self.cpp.source.components["hello"].includedirs = ["."]
                self.cpp.source.components["bye"].includedirs = ["."]
                bt = "." if self.settings.os != "Windows" else str(self.settings.build_type)
                self.cpp.build.components["hello"].libdirs = [bt]
                self.cpp.build.components["bye"].libdirs = [bt]

            def package(self):
               copy(self, "*.h", src=self.source_folder,
                                 dst=join(self.package_folder, "include"))
               copy(self, "*.lib", src=self.build_folder,
                                   dst=join(self.package_folder, "lib"), keep_path=False)
               copy(self, "*.a", src=self.build_folder,
                                 dst=join(self.package_folder, "lib"), keep_path=False)

            def package_info(self):
               self.cpp_info.components["hello"].libs = ["hello"]
               self.cpp_info.components["bye"].libs = ["bye"]

               self.cpp_info.set_property("cmake_file_name", "MYG")
               self.cpp_info.set_property("cmake_target_name", "MyGreetings::MyGreetings")
               self.cpp_info.components["hello"].set_property("cmake_target_name", "MyGreetings::MyHello")
               self.cpp_info.components["bye"].set_property("cmake_target_name", "MyGreetings::MyBye")
           """)

    cmakelists_greetings = textwrap.dedent("""
        set(CMAKE_CXX_COMPILER_WORKS 1)
        set(CMAKE_CXX_ABI_COMPILED 1)
        cmake_minimum_required(VERSION 3.0)
        project(greetings CXX)

        add_library(hello hello.cpp)
        target_include_directories(hello PRIVATE hello)
        add_library(bye bye.cpp)
        target_include_directories(bye PRIVATE bye)
        """)

    app_conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.cmake import CMake, cmake_layout

        class GreetingsTestConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeDeps", "CMakeToolchain"

            def requirements(self):
                self.requires("greetings/0.1")

            def layout(self):
                cmake_layout(self)

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()
                self.run(os.path.join(self.cpp.build.bindirs[0], "example"))
                self.run(os.path.join(self.cpp.build.bindirs[0], "example2"))
            """)
    app_cpp = gen_function_cpp(name="main", includes=["hello/hello", "bye/bye"],
                               calls=["hello", "bye"])
    app_cpp2 = gen_function_cpp(name="main", includes=["hello/hello"], calls=["hello"])

    app_cmakelists = textwrap.dedent("""
        set(CMAKE_CXX_COMPILER_WORKS 1)
        set(CMAKE_CXX_ABI_COMPILED 1)
        cmake_minimum_required(VERSION 3.0)
        project(PackageTest CXX)

        find_package(MYG)

        add_executable(example example.cpp)
        target_link_libraries(example MyGreetings::MyGreetings)

        add_executable(example2 example2.cpp)
        target_link_libraries(example2 MyGreetings::MyHello)
        """)

    client = TestClient()
    client.save({"greetings/conanfile.py": conanfile_greetings,
                 "greetings/src/CMakeLists.txt": cmakelists_greetings,
                 "greetings/src/hello/hello.h": hello_h,
                 "greetings/src/hello.cpp": hello_cpp,
                 "greetings/src/bye/bye.h": bye_h,
                 "greetings/src/bye.cpp": bye_cpp,
                 "app/conanfile.py": app_conanfile,
                 "app/example.cpp": app_cpp,
                 "app/example2.cpp": app_cpp2,
                 "app/CMakeLists.txt": app_cmakelists})
    client.run("create greetings")
    client.run("build app")
    assert "hello: Release!" in client.out
    assert "bye: Release!" in client.out

    client.run("remove * -c")
    shutil.rmtree(os.path.join(client.current_folder, "app", "build"))
    client.run("editable add greetings")
    client.run("build greetings")
    client.run("build app")
    assert str(client.out).count("hello: Release!") == 2
    assert str(client.out).count("bye: Release!") == 1

    # Do a modification to one component, to verify it is used correctly
    bye_cpp = gen_function_cpp(name="bye", includes=["bye"], msg="Adios")
    client.save({"greetings/src/bye.cpp": bye_cpp})
    client.run("build greetings")
    client.run("build app")
    assert str(client.out).count("hello: Release!") == 2
    assert str(client.out).count("Adios: Release!") == 1



import os
import pytest

# Check it is importable from tools
from conan.tools.files import rm, chdir
from conan.test.utils.test_files import temp_folder
from conan.internal.util.files import save_files


def test_remove_files_by_mask_recursively():
    temp_dir = temp_folder()

    with chdir(None, temp_dir):
        os.makedirs("subdir")
        os.makedirs("dir.pdb")
        os.makedirs(os.path.join("subdir", "deepdir"))

    save_files(temp_dir, {"1.txt": "",
                        "1.pdb": "",
                        "1.pdb1": "",
                        os.path.join("subdir", "2.txt"): "",
                        os.path.join("subdir", "2.pdb"): "",
                        os.path.join("subdir", "2.pdb1"): "",
                        os.path.join("subdir", "deepdir", "3.txt"): "",
                        os.path.join("subdir", "deepdir", "3.pdb"): "",
                        os.path.join("subdir", "deepdir", "3.pdb1"): ""})

    rm(None, "*.sh", temp_dir, recursive=True)

    rm(None, "*.pdb", temp_dir, recursive=True)

    assert os.path.isdir(os.path.join(temp_dir, "dir.pdb"))

    assert os.path.isfile(os.path.join(temp_dir, "1.txt"))
    assert not os.path.isfile(os.path.join(temp_dir, "1.pdb"))
    assert os.path.isfile(os.path.join(temp_dir, "1.pdb1"))

    assert os.path.isfile(os.path.join(temp_dir, "subdir", "2.txt"))
    assert not os.path.isfile(os.path.join(temp_dir, "subdir", "2.pdb"))
    assert os.path.isfile(os.path.join(temp_dir, "subdir", "2.pdb1"))

    assert os.path.isfile(os.path.join(temp_dir, "subdir", "deepdir", "3.txt"))
    assert not os.path.isfile(os.path.join(temp_dir, "subdir", "deepdir", "3.pdb"))
    assert os.path.isfile(os.path.join(temp_dir, "subdir", "deepdir", "3.pdb1"))

    rm(None, "*.pdb", temp_dir, recursive=True)


def test_remove_files_by_mask_non_recursively():
    temp_dir = temp_folder()
    with chdir(None, temp_dir):
        os.makedirs("subdir")

    save_files(temp_dir, {"1.txt": "",
                        "1.pdb": "",
                        "1.pdb1": "",
                        os.path.join("subdir", "2.txt"): "",
                        os.path.join("subdir", "2.pdb"): "",
                        os.path.join("subdir", "2.pdb1"): ""})

    rm(None, "*.pdb", temp_dir)
    assert not os.path.exists(os.path.join(temp_dir, "1.pdb"))
    assert os.path.exists(os.path.join(temp_dir, "subdir", "2.pdb"))

    assert os.path.exists(os.path.join(temp_dir, "1.txt"))
    assert os.path.exists(os.path.join(temp_dir, "subdir", "2.txt"))


@pytest.mark.parametrize("recursive", [False, True])
@pytest.mark.parametrize("results", [
    ["*.dll", ("foo.dll",)],
    [("*.dll",), ("foo.dll",)],
    [["*.dll"], ("foo.dll",)],
    [("*.dll", "*.lib"), ("foo.dll", "foo.dll.lib")],
])
def test_exclude_pattern_from_remove_list(recursive, results):
    """ conan.tools.files.rm should not remove files that match the pattern but are excluded
        by the excludes parameter.
        It should obey the recursive parameter, only excluding the files in the root folder in case
        it is False.
    """
    excludes, expected_files = results
    temporary_folder = temp_folder()
    with chdir(None, temporary_folder):
        os.makedirs("subdir")

    save_files(temporary_folder, {
        "1.txt": "",
        "1.pdb": "",
        "1.pdb1": "",
        "foo.dll": "",
        "foo.dll.lib": "",
        os.path.join("subdir", "2.txt"): "",
        os.path.join("subdir", "2.pdb"): "",
        os.path.join("subdir", "foo.dll"): "",
        os.path.join("subdir", "foo.dll.lib"): "",
        os.path.join("subdir", "2.pdb1"): ""})

    rm(None, "*", temporary_folder, excludes=excludes, recursive=recursive)

    for it in expected_files:
        assert os.path.exists(os.path.join(temporary_folder, it))
    assert not os.path.exists(os.path.join(temporary_folder, "1.pdb"))

    # Check the recursive parameter and subfolder
    condition = (lambda x: not x) if recursive else (lambda x: x)
    assert condition(os.path.exists(os.path.join(temporary_folder, "subdir", "2.pdb")))
    for it in expected_files:
        assert os.path.exists(os.path.join(temporary_folder, "subdir", it))



import json
import os
import textwrap

from conan.cps import CPS
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save_files


def test_cps():
    c = TestClient()
    c.save({"pkg/conanfile.py": GenConanfile("pkg", "0.1").with_settings("build_type")
                                                          .with_class_attribute("license='MIT'")})
    c.run("create pkg")

    settings = "-s os=Windows -s compiler=msvc -s compiler.version=191 -s arch=x86_64"
    c.run(f"install --requires=pkg/0.1 {settings} -g CPSDeps")
    pkg = json.loads(c.load("build/cps/msvc-191-x86_64-release/cps/pkg.cps"))
    assert pkg["name"] == "pkg"
    assert pkg["version"] == "0.1"
    assert pkg["license"] == "MIT"
    assert pkg["configurations"] == ["release"]
    assert pkg["default_components"] == ["pkg"]
    pkg_comp = pkg["components"]["pkg"]
    assert pkg_comp["type"] == "interface"
    mapping = json.loads(c.load("build/cps/cpsmap-msvc-191-x86_64-release.json"))
    for _, path_cps in mapping.items():
        assert os.path.exists(path_cps)


def test_cps_static_lib():
    c = TestClient()
    c.save({"pkg/conanfile.py": GenConanfile("pkg", "0.1").with_package_file("lib/pkg.a", "-")
                                                          .with_settings("build_type")
            .with_package_info(cpp_info={"libs": ["pkg"]}, env_info={})})
    c.run("create pkg")

    settings = "-s os=Windows -s compiler=msvc -s compiler.version=191 -s arch=x86_64"
    c.run(f"install --requires=pkg/0.1 {settings} -g CPSDeps")
    pkg = json.loads(c.load("build/cps/msvc-191-x86_64-release/cps/pkg.cps"))
    assert pkg["name"] == "pkg"
    assert pkg["version"] == "0.1"
    assert pkg["configurations"] == ["release"]
    assert pkg["default_components"] == ["pkg"]
    pkg_comp = pkg["components"]["pkg"]
    assert pkg_comp["type"] == "archive"
    assert pkg_comp["location"] is not None


def test_cps_header():
    c = TestClient()
    c.save({"pkg/conanfile.py": GenConanfile("pkg", "0.1").with_package_type("header-library")})
    c.run("create pkg")

    settings = "-s os=Windows -s compiler=msvc -s compiler.version=191 -s arch=x86_64"
    c.run(f"install --requires=pkg/0.1 {settings} -g CPSDeps")
    pkg = json.loads(c.load("build/cps/msvc-191-x86_64-release/cps/pkg.cps"))
    assert pkg["name"] == "pkg"
    assert pkg["version"] == "0.1"
    assert "configurations" not in "pkg"
    assert pkg["default_components"] == ["pkg"]
    pkg_comp = pkg["components"]["pkg"]
    assert pkg_comp["type"] == "interface"
    assert "location" not in pkg_comp


def test_cps_in_pkg():
    c = TestClient()
    cps = textwrap.dedent("""\
        {
            "cps_version": "0.12.0",
            "name": "zlib",
            "version": "1.3.1",
            "configurations": ["release"],
            "default_components": ["zlib"],
            "components": {
                "zlib": {
                    "type": "archive",
                    "includes": ["@prefix@/include"],
                    "location": "@prefix@/lib/zlib.a"
                }
            }
        }
        """)
    cps = "".join(cps.splitlines())
    conanfile = textwrap.dedent(f"""
        import os
        from conan.tools.files import save
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "zlib"
            version = "1.3.1"

            def package(self):
                cps = '{cps}'
                cps_path = os.path.join(self.package_folder, "zlib.cps")
                save(self, cps_path, cps)

            def package_info(self):
                from conan.cps import CPS
                self.cpp_info = CPS.load("zlib.cps").to_conan()
        """)
    c.save({"pkg/conanfile.py": conanfile})
    c.run("create pkg")

    settings = "-s os=Windows -s compiler=msvc -s compiler.version=191 -s arch=x86_64"
    c.run(f"install --requires=zlib/1.3.1 {settings} -g CPSDeps")

    mapping = json.loads(c.load("build/cps/cpsmap-msvc-191-x86_64-release.json"))
    for _, path_cps in mapping.items():
        assert os.path.exists(path_cps)

    assert not os.path.exists(os.path.join(c.current_folder, "zlib.cps"))
    assert not os.path.exists(os.path.join(c.current_folder, "build", "cps", "zlib.cps"))

    c.run(f"install --requires=zlib/1.3.1 {settings} -g CMakeDeps")
    cmake = c.load("zlib-release-x86_64-data.cmake")
    assert 'set(zlib_INCLUDE_DIRS_RELEASE "${zlib_PACKAGE_FOLDER_RELEASE}/include")' in cmake
    assert 'set(zlib_LIB_DIRS_RELEASE "${zlib_PACKAGE_FOLDER_RELEASE}/lib")'
    assert 'set(zlib_LIBS_RELEASE zlib)' in cmake


def test_cps_merge():
    folder = temp_folder()
    cps_base = textwrap.dedent("""
        {
          "components" : {
            "mypkg" : {
              "includes" : [ "@prefix@/include" ],
              "type" : "archive"
            }
          },
          "cps_version" : "0.12.0",
          "name" : "mypkg",
          "version" : "1.0"
        }
        """)
    cps_conf = textwrap.dedent("""
        {
          "components" : {
            "mypkg" : {
              "link_languages" : [ "cpp" ],
              "location" : "@prefix@/lib/mypkg.lib"
            }
          },
          "configuration" : "Release",
          "name" : "mypkg"
        }
        """)
    save_files(folder, {"mypkg.cps": cps_base,
                        "mypkg@release.cps": cps_conf})
    cps = CPS.load(os.path.join(folder, "mypkg.cps"))
    json_cps = cps.serialize()
    print(json.dumps(json_cps, indent=2))


def test_extended_cpp_info():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"
            def package_info(self):
                self.cpp_info.libs = ["mylib"]
                self.cpp_info.location = "my_custom_location"
                self.cpp_info.type = "static-library"
            """)
    c.save({"conanfile.py": conanfile})
    c.run("create .")

    settings = "-s os=Windows -s compiler=msvc -s compiler.version=191 -s arch=x86_64"
    c.run(f"install --requires=pkg/0.1 {settings} -g CPSDeps")
    pkg = json.loads(c.load("build/cps/msvc-191-x86_64-release/cps/pkg.cps"))
    assert pkg["name"] == "pkg"
    assert pkg["version"] == "0.1"
    assert pkg["default_components"] == ["pkg"]
    pkg_comp = pkg["components"]["pkg"]
    assert pkg_comp["type"] == "archive"
    assert pkg_comp["location"] == "my_custom_location"



import json
import platform
import textwrap
from unittest.mock import patch, MagicMock

import pytest

from conan.tools.system.package_manager import _SystemPackageManagerTool
from conan.test.utils.tools import TestClient


@pytest.mark.tool("apt_get")
@pytest.mark.skipif(platform.system() != "Linux", reason="Requires apt")
def test_apt_check():
    client = TestClient()
    client.save({"conanfile.py": textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.system.package_manager import Apt

        class MyPkg(ConanFile):
            settings = "arch", "os"
            def system_requirements(self):
                apt = Apt(self)
                not_installed = apt.check(["non-existing1", "non-existing2"])
                print("missing:", not_installed)
        """)})
    client.run("create . --name=test --version=1.0 -s:b arch=armv8 -s:h arch=x86")
    assert "missing: ['non-existing1', 'non-existing2']" in client.out


@pytest.mark.tool("apt_get")
@pytest.mark.skipif(platform.system() != "Linux", reason="Requires apt")
def test_apt_install_substitutes():
    client = TestClient()
    conanfile_py = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.system.package_manager import Apt
        class MyPkg(ConanFile):
            settings = "arch", "os"
            def system_requirements(self):
                # FIXME this is needed because the ci-functional apt-get update fails
                try:
                    self.run("sudo apt-get update")
                except Exception:
                    pass
                apt = Apt(self)
                {}
        """)

    installs = 'apt.install_substitutes(["non-existing1", "non-existing2"], ["non-existing3", "non-existing4"])'
    client.save({"conanfile.py": conanfile_py.format(installs)})
    client.run("create . --name=test --version=1.0 -c tools.system.package_manager:mode=install "
               "-c tools.system.package_manager:sudo=True", assert_error=True)
    assert "None of the installs for the package substitutes succeeded." in client.out

    client.run_command("sudo apt remove nano -yy")
    installs = 'apt.install_substitutes(["non-existing1", "non-existing2"], ["nano"], ["non-existing3"])'
    client.save({"conanfile.py": conanfile_py.format(installs)})
    client.run("create . --name=test --version=1.0 -c tools.system.package_manager:mode=install "
               "-c tools.system.package_manager:sudo=True")
    assert "1 newly installed" in client.out


@pytest.mark.tool("apt_get")
@pytest.mark.skipif(platform.system() != "Linux", reason="Requires apt")
def test_build_require():
    client = TestClient()
    client.save({"tool_require.py": textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.system.package_manager import Apt

        class MyPkg(ConanFile):
            settings = "arch", "os"
            def system_requirements(self):
                apt = Apt(self)
                not_installed = apt.check(["non-existing1", "non-existing2"])
                print("missing:", not_installed)
        """)})
    client.run("export tool_require.py --name=tool_require --version=1.0")
    client.save({"consumer.py": textwrap.dedent("""
        from conan import ConanFile
        class consumer(ConanFile):
            settings = "arch", "os"
            tool_requires = "tool_require/1.0"
        """)})
    client.run("create consumer.py --name=consumer --version=1.0 "
               "-s:b arch=armv8 -s:h arch=x86 --build=missing")
    assert "missing: ['non-existing1', 'non-existing2']" in client.out


@pytest.mark.tool("brew")
@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires brew")
def test_brew_check():
    client = TestClient()
    client.save({"conanfile.py": textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.system.package_manager import Brew

        class MyPkg(ConanFile):
            settings = "arch"
            def system_requirements(self):
                brew = Brew(self)
                not_installed = brew.check(["non-existing1", "non-existing2"])
                print("missing:", not_installed)
        """)})
    client.run("create . --name=test --version=1.0")
    assert "missing: ['non-existing1', 'non-existing2']" in client.out


@pytest.mark.tool("brew")
@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires brew")
@pytest.mark.skip(reason="brew update takes a lot of time")
def test_brew_install_check_mode():
    client = TestClient()
    client.save({"conanfile.py": textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.system.package_manager import Brew

        class MyPkg(ConanFile):
            settings = "arch"
            def system_requirements(self):
                brew = Brew(self)
                brew.install(["non-existing1", "non-existing2"])
        """)})
    client.run("create . test/1.0@", assert_error=True)
    assert "System requirements: 'non-existing1, non-existing2' are missing but " \
           "can't install because tools.system.package_manager:mode is 'check'" in client.out


@pytest.mark.tool("brew")
@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires brew")
@pytest.mark.skip(reason="brew update takes a lot of time")
def test_brew_install_install_mode():
    client = TestClient()
    client.save({"conanfile.py": textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.system.package_manager import Brew

        class MyPkg(ConanFile):
            settings = "arch"
            def system_requirements(self):
                brew = Brew(self)
                brew.install(["non-existing1", "non-existing2"])
        """)})
    client.run("create . test/1.0@ -c tools.system.package_manager:mode=install", assert_error=True)
    assert "Error: No formulae found in taps." in client.out


def test_collect_system_requirements():
    """ we can know the system_requires for every package because they are part of the graph,
    this naturally execute at ``install``, but we can also prove that with ``graph info`` we can
    for it to with the righ ``mode=collect`` mode.
    """
    client = TestClient()
    client.save({"conanfile.py": textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.system.package_manager import Brew, Apt

        class MyPkg(ConanFile):
            settings = "arch"
            def system_requirements(self):
                brew = Brew(self)
                brew.install(["brew1", "brew2"])
                apt = Apt(self)
                apt.install(["pkg1", "pkg2"])
        """)})

    with patch.object(_SystemPackageManagerTool, '_conanfile_run', MagicMock(return_value=False)):
        client.run("install . -c tools.system.package_manager:tool=apt-get --format=json",
                   redirect_stdout="graph.json")
    graph = json.loads(client.load("graph.json"))
    assert {"apt-get": {"install": ["pkg1", "pkg2"], "missing": []}} == \
           graph["graph"]["nodes"]["0"]["system_requires"]

    # plain report, do not check
    client.run("graph info . -c tools.system.package_manager:tool=apt-get "
               "-c tools.system.package_manager:mode=report --format=json",
               redirect_stdout="graph2.json")
    graph2 = json.loads(client.load("graph2.json"))
    # TODO: Unify format of ``graph info`` and ``install``
    assert {"apt-get": {"install": ["pkg1", "pkg2"]}} == \
           graph2["graph"]["nodes"]["0"]["system_requires"]

    # Check report-installed
    with patch.object(_SystemPackageManagerTool, '_conanfile_run', MagicMock(return_value=True)):
        client.run("graph info . -c tools.system.package_manager:tool=apt-get "
                   "-c tools.system.package_manager:mode=report-installed --format=json",
                   redirect_stdout="graph2.json")
        graph2 = json.loads(client.load("graph2.json"))
        assert {"apt-get": {"install": ["pkg1", "pkg2"],
                            'missing': ['pkg1', 'pkg2']}} == graph2["graph"]["nodes"]["0"]["system_requires"]

    # Default "check" will fail, as dpkg-query not installed
    client.run("graph info . -c tools.system.package_manager:tool=apt-get "
               "-c tools.system.package_manager:mode=check", assert_error=True)
    assert "ERROR: conanfile.py: Error in system_requirements() method, line 11" in client.out



import os
import shutil
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save


class TestMetadataTestPackage:
    """ It is possible to store the test_package itself in the recipe metadata and recover it
    later to execute it
    """

    def test_round_trip_with_hook(self):
        c = TestClient(default_server_user=True)
        # TODO: Better strategy for storing clean test_package, zipping it, etc
        my_hook = textwrap.dedent("""\
            import os
            from conan.tools.files import copy

            def post_export(conanfile):
               conanfile.output.info("Storing test_package")
               folder = os.path.join(conanfile.recipe_folder, "test_package")
               copy(conanfile, "*", src=folder,
                    dst=os.path.join(conanfile.recipe_metadata_folder, "test_package"))
            """)
        hook_path = os.path.join(c.paths.hooks_path, "my_hook", "hook_my_hook.py")
        save(hook_path, my_hook)
        c.save({"conanfile.py": GenConanfile("pkg", "0.1"),
                "test_package/conanfile.py": GenConanfile().with_test("pass")})
        c.run("create .")
        assert "Testing the package" in c.out

        # Now upload and remove everything
        c.run("upload * -c -r=default")
        assert "pkg/0.1: Recipe metadata: 1 files" in c.out
        c.run("remove * -c")

        c.run("install --requires=pkg/0.1 -r=default")
        # Recovery of the test package
        # TODO: Discuss if we want better UX, in a single step or something like that
        # Forcing the download of the metadata of cache-existing things with the "download" command
        c.run("download pkg/0.1 -r=default --metadata=test_package*")
        c.run("cache path pkg/0.1 --folder=metadata")
        metadata_path = str(c.stdout).strip()
        shutil.copytree(metadata_path, os.path.join(c.current_folder, "metadata"))

        # Execute the test_package
        c.run("test metadata/test_package pkg/0.1")
        assert "pkg/0.1 (test package): Running test()" in c.out



import platform
import re
import textwrap

import pytest

from conan.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() != "Windows", reason="Only for windows")
def test_nmakedeps():
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os", "arch", "compiler", "build_type"
            name = "test-nmakedeps"
            version = "1.0"

            def package_info(self):
                self.cpp_info.components["pkg-1"].libs = ["pkg-1"]
                self.cpp_info.components["pkg-1"].defines = ["TEST_DEFINITION1"]
                self.cpp_info.components["pkg-1"].system_libs = ["ws2_32"]
                self.cpp_info.components["pkg-2"].libs = ["pkg-2"]
                self.cpp_info.components["pkg-2"].defines = ["TEST_DEFINITION2=0"]
                self.cpp_info.components["pkg-2"].requires = ["pkg-1"]
                self.cpp_info.components["pkg-3"].libs = ["pkg-3"]
                self.cpp_info.components["pkg-3"].defines = ["TEST_DEFINITION3="]
                self.cpp_info.components["pkg-3"].requires = ["pkg-1", "pkg-2"]
                self.cpp_info.components["pkg-4"].libs = ["pkg-4"]
                self.cpp_info.components["pkg-4"].defines = ["TEST_DEFINITION4=foo",
                                                            "TEST_DEFINITION5=__declspec(dllexport)",
                                                            "TEST_DEFINITION6=foo bar",
                                                            "TEST_DEFINITION7=7"]
    """)
    client.save({"conanfile.py": conanfile})
    client.run("create . -s arch=x86_64")
    client.run("install --requires=test-nmakedeps/1.0"
               " -g NMakeDeps -s build_type=Release -s arch=x86_64")
    # Checking that NMakeDeps builds correctly .bat file
    bat_file = client.load("conannmakedeps.bat")
    # Checking that defines are added to CL
    for flag in (
        r'/D"TEST_DEFINITION1"', '/D"TEST_DEFINITION2#0"',
        r'/D"TEST_DEFINITION3#"', '/D"TEST_DEFINITION4#"foo""',
        r'/D"TEST_DEFINITION5#"__declspec\(dllexport\)""',
        r'/D"TEST_DEFINITION6#"foo bar""',
        r'/D"TEST_DEFINITION7#7"'
    ):
        assert re.search(fr'set "CL=%CL%.*\s{flag}(?:\s|")', bat_file)
    # Checking that libs and system libs are added to _LINK_
    for flag in (r"pkg-1\.lib", r"pkg-2\.lib", r"pkg-3\.lib", r"pkg-4\.lib", r"ws2_32\.lib"):
        assert re.search(fr'set "_LINK_=%_LINK_%.*\s{flag}(?:\s|")', bat_file)



import os
import shutil
from os.path import join

import pytest

from conan.errors import ConanException
from conan.test.utils.test_files import temp_folder
from conan.internal.util.files import mkdir, save, merge_directories, load


class TestMergeDirectories:

    @pytest.fixture(autouse=True)
    def setup(self):
        self.source = temp_folder()
        self.dest = temp_folder()

    def _save(self, folder, files, content=None):
        for the_file in files:
            save(join(folder, the_file), content or "c")

    def _get_paths(self, folder):
        ret = []
        for root, dirs, files in os.walk(folder):
            for the_file in files:
                relpath = os.path.relpath(root, folder)
                if relpath == ".":
                    relpath = ""
                ret.append(join(relpath, the_file))
            for the_dir in dirs:
                if not os.listdir(join(root, the_dir)):
                    relpath = os.path.relpath(join(root, the_dir), folder)
                    ret.append(relpath)
        return ret

    def _assert_equals(self, list1, list2):
        assert set([el.replace("/", "\\") for el in list1]) == set([el.replace("/", "\\") for el in list2])

    def test_empty_dest_merge(self):
        files = ["file.txt", "subdir/file2.txt"]
        self._save(self.source, files)
        merge_directories(self.source, self.dest)
        self._assert_equals(self._get_paths(self.dest), files)

    def test_non_empty_dest_merge(self):
        files = ["file.txt", "subdir/file2.txt"]
        self._save(self.source, files, "fromsrc")

        files_dest = ["file.txt", "subdir2/file2.txt"]
        self._save(self.dest, files_dest, "fromdest")

        merge_directories(self.source, self.dest)
        self._assert_equals(self._get_paths(self.dest), files + files_dest)
        # File from src overrides file from dest
        assert load(join(self.dest, "file.txt")) == "fromsrc"

    def test_nested_directories(self):
        self.dest = join(self.source, "destination_dir")
        files_dest = ["file.txt", "subdir2/file2.txt"]
        self._save(self.dest, files_dest, "fromdest")
        mkdir(join(self.dest, "empty_folder", "subempty_folder"))

        files = ["file.txt", "subdir/file2.txt"]
        self._save(self.source, files, "fromsrc")

        merge_directories(self.source, self.dest)
        self._assert_equals(self._get_paths(self.dest), files + files_dest +
                            ['empty_folder/subempty_folder', ])
        assert load(join(self.dest, "file.txt")) == "fromsrc"
        assert load(join(self.dest, "subdir2/file2.txt")) == "fromdest"
        assert load(join(self.dest, "subdir/file2.txt")) == "fromsrc"

    def test_same_directory(self):
        # Same directory cannot be merged, this should never happen
        with pytest.raises(ConanException):
            merge_directories(self.source, self.source)

    def test_parent_directory(self):
        files_dest = ["file.txt", "subdir2/file2.txt"]
        self._save(self.dest, files_dest, "fromdest")
        self.source = join(self.dest, "source_folder")
        files = ["file.txt", "subdir/file2.txt"]
        self._save(self.source, files, "fromsrc")
        merge_directories(self.source, self.dest)
        shutil.rmtree(self.source)
        self._assert_equals(self._get_paths(self.dest), files + files_dest)
        assert load(join(self.dest, "file.txt")) == "fromsrc"
        assert load(join(self.dest, "subdir2/file2.txt")) == "fromdest"
        assert load(join(self.dest, "subdir/file2.txt")) == "fromsrc"



import pytest

from conan.internal.model.version import Version

v = [("1.2.3",
      (1, 2, 3), None, None),
     ("1.2.3",
      ('1', '2', '3'), None, None),
     ("master+build2",
      'master', None, "build2"),
     ("1.2.3-alpha1+build2",
      (1, 2, 3), "alpha1", "build2"),
     ("1.2.3+build2",
      (1, 2, 3), None, "build2"),
     ("1.2.3b.4-pre.1.2b+build.1.1b",
      (1, 2, "3b", 4), "pre.1.2b", "build.1.1b"),
     ("0.2.3+b178",
      (0, 2, 3), None, "b178"),
     ("1.2.3.4.5",
      (1, 2, 3, 4, 5), None, None),
     (9,
      9, None, None),
     (1.2,
      (1, 2), None, None),
     ("",
      '', None, None),
     ("+build2",
      '', None, "build2"),
     ("1.2.3-pre.4-5-6+build.7-8-9",
      "1.2.3", "pre.4-5-6", "build.7-8-9"),
     ("1-", "1", "", None)
     ]


@pytest.mark.parametrize("v_str, main, pre, build", v)
def test_parse(v_str, main, pre, build):
    v1 = Version(v_str)
    assert v1.main == main if type(main) is tuple else tuple([main])
    assert v1.pre == pre
    assert v1.build == build
    assert str(v1) == str(v_str)



import os
import textwrap

from conan.api.model import RecipeReference
from conan.test.utils.tools import TestClient, GenConanfile


class TestConanAlias:

    def test_repeated_alias(self):
        client = TestClient(light=True)
        client.alias("hello/0.x@lasote/channel",  "hello/0.1@lasote/channel")
        client.alias("hello/0.x@lasote/channel",  "hello/0.2@lasote/channel")
        client.alias("hello/0.x@lasote/channel",  "hello/0.3@lasote/channel")

    def test_basic(self):
        client = TestClient(light=True, default_server_user=True)
        for i in (1, 2):
            client.save({"conanfile.py": GenConanfile().with_name("hello").with_version("0.%s" % i)})
            client.run("export . --user=lasote --channel=channel")

        client.alias("hello/0.x@lasote/channel",  "hello/0.1@lasote/channel")
        conanfile_chat = textwrap.dedent("""
            from conan import ConanFile
            class TestConan(ConanFile):
                name = "chat"
                version = "1.0"
                requires = "hello/(0.x)@lasote/channel"
                """)
        client.save({"conanfile.py": conanfile_chat}, clean_first=True)
        client.run("export . --user=lasote --channel=channel")
        client.save({"conanfile.txt": "[requires]\nchat/1.0@lasote/channel"}, clean_first=True)

        client.run("install . --build=missing")
        assert ("chat/1.0@lasote/channel: WARN: legacy: Requirement 'alias' is provided in Conan 2"
                in client.out)

        client.assert_listed_require({"hello/0.1@lasote/channel": "Cache"})
        assert "hello/0.x@lasote/channel: hello/0.1@lasote/channel" in client.out

        ref = RecipeReference.loads("chat/1.0@lasote/channel")
        pref = client.get_latest_package_reference(ref)
        pkg_folder = client.get_latest_pkg_layout(pref).package()
        conaninfo = client.load(os.path.join(pkg_folder, "conaninfo.txt"))

        assert "hello/0.1" in conaninfo
        assert "hello/0.x" not in conaninfo

        client.run('upload "*" --confirm -r default')
        client.run('remove "*" -c')

        client.run("install .")
        assert "'alias' is a Conan 1.X legacy feature" in client.out
        client.assert_listed_require({"hello/0.1@lasote/channel": "Downloaded (default)"})
        assert "hello/0.x@lasote/channel from" not in client.out

        client.alias("hello/0.x@lasote/channel",  "hello/0.2@lasote/channel")
        client.run("install . --build=missing")
        assert "hello/0.2" in client.out
        assert "hello/0.1" not in client.out

    def test_not_override_package(self):
        """ Do not override a package with an alias

            If we create an alias with the same name as an existing package, it will
            override the package without any warning.
        """
        t = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                description = "{}"
            """)

        # Create two packages
        reference1 = "pkga/0.1@user/testing"
        t.save({"conanfile.py": conanfile.format(reference1)})
        t.run("export . --name=pkga --version=0.1 --user=user --channel=testing")

        reference2 = "pkga/0.2@user/testing"
        t.save({"conanfile.py": conanfile.format(reference2)})
        t.run("export . --name=pkga --version=0.2 --user=user --channel=testing")



import os

from conan.api.conan_api import ConanAPI
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.test_files import temp_folder
from conan.api.model import RecipeReference
from conan.internal.util.files import save


def test_local_api():
    # https://github.com/conan-io/conan/issues/17484
    current_folder = temp_folder()
    cache_folder = temp_folder()
    save(os.path.join(current_folder, "conanfile.py"), str(GenConanfile("foo", "1.0")))
    api = ConanAPI(cache_folder)
    assert api.local.editable_packages.edited_refs == {}
    api.local.editable_add(".", cwd=current_folder)
    assert list(api.local.editable_packages.edited_refs) == [RecipeReference.loads("foo/1.0")]



import os
from unittest.mock import MagicMock

import pytest

from conan.api.conan_api import ConanAPI
from conan.internal.cache.home_paths import HomePaths
from conan.errors import ConanException
from conan.test.utils.test_files import temp_folder
from conan.internal.util.files import save


@pytest.fixture()
def conan_api():
    tmp_folder = temp_folder()
    home_path = HomePaths(tmp_folder)
    save(os.path.join(home_path.profiles_path, "default"), "")
    return ConanAPI(tmp_folder)


@pytest.fixture()
def argparse_args():
    return MagicMock(
        profile_build=None,
        profile_host=None,
        profile_all=None,
        settings_build=None,
        settings_host=None,
        settings_all=None,
        options_build=None,
        options_host=None,
        options_all=None,
        conf_build=None,
        conf_host=None,
        conf_all=None,
    )


@pytest.mark.parametrize("conf_name", [
    "core.doesnotexist:never",
    "core:doesnotexist"
])
def test_core_confs_not_allowed_via_cli(conan_api, argparse_args, conf_name):
    argparse_args.conf_build = [conf_name]
    argparse_args.conf_host = [conf_name]

    with pytest.raises(ConanException) as exc:
        conan_api.profiles.get_profiles_from_args(argparse_args)
    assert "[conf] 'core.*' configurations are not allowed in profiles" in str(exc.value)



import textwrap

from jinja2 import Template


def gen_premake5(workspace, projects, includedirs=None, configurations=None):
    includedirs = includedirs if includedirs is not None else ["."]
    premake5 = textwrap.dedent("""\
        workspace "{{workspace}}"
            cppdialect "C++17"
            configurations {{premake_quote(configurations)}}
            fatalwarnings {"All"}
            floatingpoint "Fast"
            includedirs {{premake_quote(includedirs)}}
            filter "configurations:Debug"
               defines { "DEBUG" }
               symbols "On"

            filter "configurations:Release"
               defines { "NDEBUG" }
               optimize "On"

        {% for project in projects %}
        project "{{project["name"]}}"
            kind "{{project["kind"] or 'StaticLib'}}"
            language "{{project["language"] or 'C++'}}"
            files {{premake_quote(project["files"])}}
            links {{premake_quote(project["links"])}}
        {% endfor %}
        """)

    t = Template(premake5, trim_blocks=True, lstrip_blocks=True)

    def premake_quote(s):
        return '{' + ', '.join(['"{}"'.format(s) for s in s]) + '}'

    return t.render(
        {
            "premake_quote": premake_quote,
            "workspace": workspace,
            "configurations": configurations or ["Debug", "Release"],
            "includedirs": includedirs,
            "projects": projects,
        }
    )



import platform
import textwrap

import pytest

from test.conftest import tools_locations
from conan.test.utils.tools import TestClient


@pytest.mark.tool("pkg_config")
class TestPkgConfig:
    """ This test uses the pkg_config in the system
    """
    def test_negative(self):
        c = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.gnu import PkgConfig

            class Pkg(ConanFile):
                def generate(self):
                    pkg_config = PkgConfig(self, "something_that_not_exist")
                    pkg_config.libs
                """)
        c.save({"conanfile.py": conanfile})
        c.run("install .", assert_error=True)
        assert "PkgConfig failed. Command: pkg-config --libs-only-l something_that_not_exist " \
               "--print-errors" in c.out
        assert "Package something_that_not_exist was not found" in c.out

    def test_pc(self):
        c = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.gnu import PkgConfig
            from conan.tools import CppInfo

            class Pkg(ConanFile):
                def generate(self):
                    pkg_config = PkgConfig(self, "libastral", pkg_config_path=".")
                    self.output.info(f"PROVIDES: {pkg_config.provides}")
                    self.output.info(f"VERSION: {pkg_config.version}")
                    self.output.info(f"VARIABLES: {pkg_config.variables['prefix']}")

                    cpp_info = CppInfo(self)
                    pkg_config.fill_cpp_info(cpp_info, is_system=False, system_libs=["m"])

                    assert cpp_info.includedirs == ['/usr/local/include/libastral']
                    assert cpp_info.defines == ['_USE_LIBASTRAL']
                    assert cpp_info.libs == ['astral']
                    assert cpp_info.system_libs == ['m']
                    assert cpp_info.libdirs == ['/usr/local/lib/libastral']
                    assert cpp_info.sharedlinkflags == ['-Wl,--whole-archive']
            """)
        libastral_pc = textwrap.dedent("""\
            PC FILE EXAMPLE:

            prefix=/usr/local
            exec_prefix=${prefix}
            libdir=${exec_prefix}/lib
            includedir=${prefix}/include

            Name: libastral
            Description: Interface library for Astral data flows
            Version: 6.6.6
            Libs: -L${libdir}/libastral -lastral -lm -Wl,--whole-archive
            Cflags: -I${includedir}/libastral -D_USE_LIBASTRAL
            """)
        c.save({"conanfile.py": conanfile,
                "libastral.pc": libastral_pc})
        c.run("install .")
        assert "conanfile.py: PROVIDES: libastral = 6.6.6" in c.out
        assert "conanfile.py: VERSION: 6.6.6" in c.out
        assert "conanfile.py: VARIABLES: /usr/local" in c.out


def test_pkg_config_round_tripe_cpp_info():
    """ test that serialize and deserialize CppInfo works
    """
    try:
        version = tools_locations["pkg_config"]["default"]
        exe = tools_locations["pkg_config"]["exe"]
        os_ = platform.system()
        pkg_config_path = tools_locations["pkg_config"][version]["path"][os_] + "/" + exe
    except KeyError:
        pytest.skip("pkg-config path not defined")
        return

    c = TestClient()
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.gnu import PkgConfig
        from conan.tools import CppInfo

        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"
            exports_sources = "*.pc"

            def package(self):
                pkg_config = PkgConfig(self, "libastral", pkg_config_path=".")
                cpp_info = CppInfo(self)
                pkg_config.fill_cpp_info(cpp_info, is_system=False, system_libs=["m"])
                cpp_info.save(os.path.join(self.package_folder, "cpp_info.json"))

            def package_info(self):
                self.cpp_info = CppInfo(self).load("cpp_info.json")
        """)
    prefix = "C:" if platform.system() == "Windows" else ""
    libastral_pc = textwrap.dedent("""\
        PC FILE EXAMPLE:

        prefix=%s/usr/local
        exec_prefix=${prefix}
        libdir=${exec_prefix}/lib
        includedir=${prefix}/include

        Name: libastral
        Description: Interface library for Astral data flows
        Version: 6.6.6
        Libs: -L${libdir}/libastral -lastral -lm -Wl,--whole-archive
        Cflags: -I${includedir}/libastral -D_USE_LIBASTRAL
        """ % prefix)
    c.save({"conanfile.py": conanfile,
            "libastral.pc": libastral_pc,
            "profile": f"[conf]\ntools.gnu:pkg_config={pkg_config_path}"})
    c.run("export .")
    c.run("install --requires=pkg/0.1 -pr=profile -g CMakeDeps --build=missing")
    pkg_data = c.load("pkg-none-data.cmake")
    assert 'set(pkg_DEFINITIONS_NONE "-D_USE_LIBASTRAL")' in pkg_data
    assert 'set(pkg_SHARED_LINK_FLAGS_NONE "-Wl,--whole-archive")' in pkg_data
    assert 'set(pkg_COMPILE_DEFINITIONS_NONE "_USE_LIBASTRAL")' in pkg_data
    assert 'set(pkg_LIBS_NONE astral)' in pkg_data
    assert 'set(pkg_SYSTEM_LIBS_NONE m)' in pkg_data
    # paths
    assert f'set(pkg_INCLUDE_DIRS_NONE "{prefix}/usr/local/include/libastral")' in pkg_data
    assert f'set(pkg_LIB_DIRS_NONE "{prefix}/usr/local/lib/libastral")' in pkg_data



import json
import textwrap

from conan.test.utils.tools import TestClient, GenConanfile


class TestPerPackageSetting:

    def test_per_package_setting(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                settings = "os"
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . --name=pkg --version=0.1 --user=user --channel=testing -s os=Windows")
        client.save({"conanfile.py": GenConanfile().with_require("pkg/0.1@user/testing")})
        client.run("create . --name=consumer --version=0.1 --user=user --channel=testing -s os=Linux -s pkg*:os=Windows")
        assert "consumer/0.1@user/testing: Created package" in client.out

    def test_per_package_setting_build_type(self):
        """ comes from https://github.com/conan-io/conan/pull/9842
        In Conan 1.X there was a weird behavior if you used different patterns,
        only first matching one win. This was broken:
          -s pkg*:os=Windows -s *model:build_type=Debug
        keeps the build_type=Release, because pkg* matches, assign os=Windows and stops
        """
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                name = "pkg-model"
                version = "0.1"
                settings = "os", "build_type"
                def build(self):
                    self.output.info("BUILDTYPE {}:{}!!!!".format(self.settings.os,
                                                                  self.settings.build_type))
            """)
        client.save({"conanfile.py": conanfile})
        # Different pattern, no issue, works now
        client.run("create . -s *:os=Windows -s *-model*:build_type=Debug -s build_type=Release")
        assert "pkg-model/0.1: BUILDTYPE Windows:Debug!!!!" in client.out

        client.save({"conanfile.py": GenConanfile("consumer", "0.1").with_require("pkg-model/0.1")})
        client.run('create . -s *:os=Linux -s *-model*:os=Windows '
                   '-s "*-model*:build_type=Debug" -s build_type=Release --build=*')
        assert "pkg-model/0.1: BUILDTYPE Windows:Debug!!!!" in client.out
        assert "consumer/0.1: Created package" in client.out

    def test_per_package_setting_no_userchannel(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                settings = "os"
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . --name=pkg --version=0.1 -s os=Windows")
        client.save({"conanfile.py": GenConanfile().with_require("pkg/0.1")})
        client.run("create . --name=consumer --version=0.1 -s os=Linux -s pkg*:os=Windows")
        assert "consumer/0.1: Created package" in client.out

    def test_per_package_subsetting(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                settings = "os", "compiler"
            """)
        client.save({"conanfile.py": conanfile})
        settings = "-s os=Linux -s compiler=gcc -s compiler.version=5"
        client.run("create . --name=pkg --version=0.1 --user=user --channel=testing %s  -s compiler.libcxx=libstdc++11" % settings)
        client.save({"conanfile.py": GenConanfile().with_require("pkg/0.1@user/testing")})
        client.run("create . --name=consumer --version=0.1 --user=user --channel=testing %s -s compiler.libcxx=libstdc++ "
                   "-s pkg*:compiler.libcxx=libstdc++11" % settings)
        assert "consumer/0.1@user/testing: Created package" in client.out

    def test_per_package_setting_all_packages_without_user_channel(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                settings = "os"
                def configure(self):
                    self.output.info(f"I am a {self.settings.os} pkg!!!")
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . --name=pkg1 --version=0.1 -s os=Windows")
        client.run("create . --name=pkg2 --version=0.1 --user=user -s os=Linux")
        client.run("create . --name=pkg3 --version=0.1 --user=user --channel=channel -s os=Linux")
        client.save({"conanfile.py": GenConanfile().with_requires("pkg1/0.1", "pkg2/0.1@user",
                                                                  "pkg3/0.1@user/channel")})
        client.run("install . -s os=Linux -s *@:os=Windows")
        assert "pkg1/0.1: I am a Windows pkg!!!" in client.out
        assert "pkg2/0.1@user: I am a Linux pkg!!!" in client.out
        assert "pkg3/0.1@user/channel: I am a Linux pkg!!!" in client.out


def test_per_package_settings_target():
    c = TestClient()
    gcc = textwrap.dedent("""
        from conan import ConanFile

        class Pkg(ConanFile):
            name = "gcc"
            version = "0.1"
            settings = "arch"

            def configure(self):
                self.settings_target.rm_safe("os")
                self.settings_target.rm_safe("compiler")
                self.settings_target.rm_safe("build_type")

            def package_id(self):
                self.info.settings_target = self.settings_target
        """)
    c.save({"conanfile.py": gcc})
    c.run("create . -s:b arch=x86_64 -s:h arch=armv7 --build-require")
    pkg_id = c.created_package_id("gcc/0.1")

    c.run("install --tool-requires=gcc/0.1 -s:b arch=x86_64 -s arch=armv8 -s:h gcc*:arch=armv7"
          " --format=json")
    # it will not fail due to armv8, but use the binary for armv7
    c.assert_listed_binary({"gcc/0.1": (pkg_id, "Cache")}, build=True)

    graph = json.loads(c.stdout)
    assert graph["graph"]["nodes"]["1"]["info"]["settings_target"] == {"arch": "armv7"}


def test_per_package_settings_build():
    c = TestClient()
    cmake = textwrap.dedent("""
        from conan import ConanFile

        class Pkg(ConanFile):
            name = "cmake"
            version = "0.1"
            settings = "build_type"

            def build(self):
                self.output.info(f"Building myself with {self.settings.build_type}!!")
        """)
    c.save({"conanfile.py": cmake})
    c.run("export .")

    c.run("install --tool-requires=cmake/0.1 -s:b build_type=Release -s:b cmake*:build_type=Debug "
          "--build=missing")
    assert "cmake/0.1: Building myself with Debug!!" in c.out


def test_package_settings_mixed_patterns():
    # https://github.com/conan-io/conan/issues/16606
    c = TestClient()
    profile = textwrap.dedent("""
        [settings]
        arch=x86_64
        *@test/*:build_type=Release

        os=Linux
        compiler=gcc
        &:compiler.version=12
        compiler.libcxx = libstdc++
        """)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "mypkg"
            version = "0.1"
            settings = "os", "arch", "compiler", "build_type"
            def build(self):
                self.output.info(f"BUILD_TYPE={self.settings.build_type}!")
                self.output.info(f"COMPILER_VERSION={self.settings.compiler.version}!")
                self.output.info(f"COMPILER_LIBCXX={self.settings.compiler.libcxx}!")
            """)
    c.save({"profile": profile,
            "conanfile.py": conanfile})
    c.run("create . -pr=profile --user=test --channel=test")
    assert "mypkg/0.1@test/test: BUILD_TYPE=Release!" in c.out
    assert "mypkg/0.1@test/test: COMPILER_VERSION=12!" in c.out
    assert "mypkg/0.1@test/test: COMPILER_LIBCXX=libstdc++!" in c.out


def test_package_settings_negate_patterns():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Dep(ConanFile):
            version = "0.1"
            settings = "os"
            def build(self):
                self.output.info(f"BUILD OS={self.settings.os}!!!")
            """)
    c.save({"dep/conanfile.py": conanfile,
            "pkg/conanfile.py": GenConanfile().with_requires("dep1/0.1", "dep2/0.1", "dep3/0.1")})
    c.run("export dep --name=dep1")
    c.run("export dep --name=dep2")
    c.run("export dep --name=dep3")

    c.run("install pkg --build=* -s os=Linux -s ~dep1/*:os=Macos")
    assert "dep1/0.1: BUILD OS=Linux!!!" in c.out
    assert "dep2/0.1: BUILD OS=Macos!!!" in c.out
    assert "dep3/0.1: BUILD OS=Macos!!!" in c.out

    # Order doesn't matter if using the base "os=Linux"
    c.run("install pkg --build=* -s ~dep1/*:os=Macos -s os=Linux")
    assert "dep1/0.1: BUILD OS=Linux!!!" in c.out
    assert "dep2/0.1: BUILD OS=Macos!!!" in c.out
    assert "dep3/0.1: BUILD OS=Macos!!!" in c.out

    # Order DOES matter if using the pattern "*:os=Linux"
    c.run("install pkg --build=* -s ~dep1/*:os=Macos -s *:os=Linux")
    assert "dep1/0.1: BUILD OS=Linux!!!" in c.out
    assert "dep2/0.1: BUILD OS=Linux!!!" in c.out
    assert "dep3/0.1: BUILD OS=Linux!!!" in c.out

    # dep3 comes later, works
    c.run("install pkg --build=* -s os=Linux -s ~dep1/*:os=Macos -s dep3/*:os=Windows")
    assert "dep1/0.1: BUILD OS=Linux!!!" in c.out
    assert "dep2/0.1: BUILD OS=Macos!!!" in c.out
    assert "dep3/0.1: BUILD OS=Windows!!!" in c.out

    # dep3 comes first, then last !dep1 pattern prevails
    c.run("install pkg --build=* -s os=Linux -s dep3/*:os=Windows -s ~dep1/*:os=Macos")
    assert "dep1/0.1: BUILD OS=Linux!!!" in c.out
    assert "dep2/0.1: BUILD OS=Macos!!!" in c.out
    assert "dep3/0.1: BUILD OS=Macos!!!" in c.out



from conan.tools.system import PipEnv
from unittest.mock import patch
import pytest
from conan.errors import ConanException
from conan.internal.model.settings import Settings
from conan.test.utils.mocks import ConanFileMock


@patch('shutil.which')
def test_pipenv_conf(mock_shutil_which):
    conanfile = ConanFileMock()
    conanfile.settings = Settings()
    conanfile.conf.define("tools.system.pipenv:python_interpreter",
                          "/python/interpreter/from/config")
    result = "/python/interpreter/from/config -m venv"
    PipEnv(conanfile, "testenv")

    def fake_run(command, win_bash=False, subsystem=None, env=None, ignore_errors=False,   # noqa
                 quiet=False):  # noqa
        assert result in command
        return 100
    conanfile.run = fake_run
    mock_shutil_which.assert_not_called()


@patch('shutil.which')
def test_pipenv_error_message(mock_shutil_which):
    conanfile = ConanFileMock()
    conanfile.settings = Settings()
    mock_shutil_which.return_value = None
    with pytest.raises(ConanException) as exc_info:
        PipEnv(conanfile, "testenv")
    assert ("install Python system-wide or set the 'tools.system.pipenv:python_interpreter' "
            "conf") in exc_info.value.args[0]


def test_pipenv_creation_error_message():
    conanfile = ConanFileMock()
    conanfile.settings = Settings()
    conanfile.conf.define("tools.system.pipenv:python_interpreter",
                          "/python/interpreter/from/config")

    def fake_run(command, win_bash=False, subsystem=None, env=None, ignore_errors=False,   # noqa
                 quiet=False):  # noqa
        raise ConanException("fake error message")
    conanfile.run = fake_run
    with pytest.raises(ConanException) as exc_info:
        PipEnv(conanfile, "testenv")
    assert "using '/python/interpreter/from/config': fake error message" in exc_info.value.args[0]



import textwrap

from conan.test.utils.tools import TestClient


class TestAttributesScope:

    def test_cppinfo_not_in_package_id(self):
        # self.cpp_info is not available in 'package_id'
        t = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile

            class Recipe(ConanFile):

                def package_id(self):
                    self.cpp_info.libs = ["A"]
        """)
        t.save({'conanfile.py': conanfile})
        t.run('create . --name=name --version=version', assert_error=True)
        assert "'self.cpp_info' access in 'package_id()' method is forbidden" in t.out

    def test_settings_not_in_package_id(self):
        # self.cpp_info is not available in 'package_id'
        t = TestClient()
        conanfile = textwrap.dedent("""
               from conan import ConanFile

               class Recipe(ConanFile):

                   def package_id(self):
                       self.settings
           """)
        t.save({'conanfile.py': conanfile})
        t.run('create . --name=name --version=version', assert_error=True)
        assert "'self.settings' access in 'package_id()' method is forbidden" in t.out

    def test_options_not_in_package_id(self):
        # self.cpp_info is not available in 'package_id'
        t = TestClient()
        conanfile = textwrap.dedent("""
               from conan import ConanFile

               class Recipe(ConanFile):

                   def package_id(self):
                       self.options
           """)
        t.save({'conanfile.py': conanfile})
        t.run('create . --name=name --version=version', assert_error=True)
        assert "'self.options' access in 'package_id()' method is forbidden" in t.out

    def test_info_not_in_package_info(self):
        t = TestClient()
        conanfile = textwrap.dedent("""
               from conan import ConanFile

               class Recipe(ConanFile):

                   def package_info(self):
                       self.info
           """)
        t.save({'conanfile.py': conanfile})
        t.run('create . --name=name --version=version', assert_error=True)
        assert "'self.info' access in 'package_info()' method is forbidden" in t.out

    def test_info_not_in_package(self):
        # self.info is not available in 'package'
        t = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile

            class Recipe(ConanFile):

                def package(self):
                    self.info.clear()
        """)
        t.save({'conanfile.py': conanfile})
        t.run('create . --name=name --version=version -s os=Linux', assert_error=True)
        assert "'self.info' access in 'package()' method is forbidden" in t.out

    def test_no_settings(self):
        # self.setting is not available in 'source'
        t = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile

            class Recipe(ConanFile):
                settings = "os",

                def source(self):
                    self.settings.os
        """)
        t.save({'conanfile.py': conanfile})
        t.run('create . --name=name --version=version -s os=Linux', assert_error=True)
        assert "'self.settings' access in 'source()' method is forbidden" in t.out

    def test_no_options(self):
        # self.setting is not available in 'source'
        t = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile

            class Recipe(ConanFile):
                options = {'shared': [True, False]}

                def source(self):
                    self.options.shared
        """)
        t.save({'conanfile.py': conanfile})
        t.run('create . --name=name --version=version -o shared=False', assert_error=True)
        assert "'self.options' access in 'source()' method is forbidden" in t.out



import os
import textwrap


from conan.test.utils.tools import TestClient


class TestSettingsLoad:
    def test_invalid_settings(self):
        client = TestClient()
        client.save({os.path.join(client.cache_folder, 'settings.yml'): "your buggy file",
                     "conanfile.txt": ""})
        client.run("install .", assert_error=True)
        assert "ERROR: Invalid settings.yml format: 'settings' is not a dictionary" in client.out
        client.save({os.path.join(client.cache_folder, 'settings.yml'): "os:\n    Windows: [1, ]",
                     "conanfile.txt": ""})
        client.run("install .", assert_error=True)
        assert "ERROR: Invalid settings.yml format: 'settings.os=Windows' is not a dictionary" \
               in client.out

    def test_invalid_yaml(self):
        client = TestClient()
        client.save({os.path.join(client.cache_folder, 'settings.yml'):
                     textwrap.dedent("""
                        Almost:
                            - a
                            - valid
                          yaml
                     """),
                     "conanfile.txt": ""})
        client.run("install .", assert_error=True)
        assert "ERROR: Invalid settings.yml format: while parsing a block mapping" in client.out



import os
import platform
import subprocess
import textwrap

import pytest

from conan.errors import ConanException
from conan.tools.env import Environment
from conan.tools.env.environment import ProfileEnvironment
from conan.internal.subsystems import WINDOWS
from conan.api.model import RecipeReference
from conan.test.utils.mocks import ConanFileMock, MockSettings
from conan.test.utils.test_files import temp_folder
from conan.test.utils.env import environment_update
from conan.internal.util.files import save, chdir


def test_compose():
    env = Environment()
    env.define("MyVar", "MyValue")
    env.define("MyVar2", "MyValue2")
    env.define("MyVar3", "MyValue3")
    env.define("MyVar4", "MyValue4")
    env.unset("MyVar5")

    env2 = Environment()
    env2.define("MyVar", "MyNewValue")
    env2.append("MyVar2", "MyNewValue2")
    env2.prepend("MyVar3", "MyNewValue3")
    env2.unset("MyVar4")
    env2.define("MyVar5", "MyNewValue5")

    env.compose_env(env2)
    env = env.vars(ConanFileMock())
    assert env.get("MyVar") == "MyValue"
    assert env.get("MyVar2") == 'MyValue2'
    assert env.get("MyVar3") == 'MyValue3'
    assert env.get("MyVar4") == "MyValue4"
    assert env.get("MyVar5") == ''


def test_define_append():
    env = Environment()
    env.define("MyVar", "MyValue")
    env.append("MyVar", "MyValue1")
    env.append("MyVar", ["MyValue2", "MyValue3"])
    assert env.vars(ConanFileMock()).get("MyVar") == "MyValue MyValue1 MyValue2 MyValue3"

    env = Environment()
    env.append("MyVar", "MyValue")
    env.append("MyVar", "MyValue1")
    env.define("MyVar", "MyValue2")
    assert env.vars(ConanFileMock()).get("MyVar") == "MyValue2"


@pytest.mark.parametrize("op1, v1, s1, op2, v2, s2, result",
                         [("define", "Val1", " ", "define", "Val2", " ", "Val1"),
                          ("define", "Val1", " ", "append", "Val2", " ", "Val1"),
                          ("define", "Val1", " ", "prepend", "Val2", " ", "Val1"),
                          ("define", "Val1", " ", "unset", "", " ", "Val1"),
                          ("append", "Val1", " ", "define", "Val2", " ", "Val2 Val1"),
                          ("append", "Val1", " ", "append", "Val2", " ", "MyVar Val2 Val1"),
                          ("append", "Val1", " ", "prepend", "Val2", " ", "Val2 MyVar Val1"),
                          ("append", "Val1", " ", "unset", "", " ", "Val1"),
                          ("prepend", "Val1", " ", "define", "Val2", " ", "Val1 Val2"),
                          ("prepend", "Val1", " ", "append", "Val2", " ", "Val1 MyVar Val2"),
                          ("prepend", "Val1", " ", "prepend", "Val2", " ", "Val1 Val2 MyVar"),
                          ("prepend", "Val1", " ", "unset", "", " ", "Val1"),
                          ("unset", "", " ", "define", "Val2", " ", ""),
                          ("unset", "", " ", "append", "Val2", " ", ""),
                          ("unset", "", " ", "prepend", "Val2", " ", ""),
                          ("unset", "", " ", "unset", "", " ", ""),
                          # different separators
                          ("append", "Val1", "+", "append", "Val2", "-", "MyVar+Val2+Val1"),
                          ("append", "Val1", "+", "prepend", "Val2", "-", "Val2+MyVar+Val1"),
                          ("unset", "", " ", "append", "Val2", "+", ""),
                          ("unset", "", " ", "prepend", "Val2", "+", ""),
                          ])
def test_compose_combinations(op1, v1, s1, op2, v2, s2, result):
    env = Environment()
    if op1 != "unset":
        getattr(env, op1)("MyVar", v1, s1)
    else:
        env.unset("MyVar")
    env2 = Environment()
    if op2 != "unset":
        getattr(env2, op2)("MyVar", v2, s2)
    else:
        env2.unset("MyVar")
    env.compose_env(env2)
    env = env.vars(ConanFileMock())
    with environment_update({"MyVar": "MyVar"}):
        assert env.get("MyVar") == result
    assert env._values["MyVar"].get_str("{name}", None, None) == result


@pytest.mark.parametrize("op1, v1, op2, v2, result",
                         [("define", "/path1", "define", "/path2", "/path1"),
                          ("define", "/path1", "append", "/path2", "/path1"),
                          ("define", "/path1", "prepend", "/path2", "/path1"),
                          ("define", "/path1", "unset", "", "/path1"),
                          ("append", "/path1", "define", "/path2", "/path2:/path1"),
                          ("append", "/path1", "append", "/path2", "MyVar:/path2:/path1"),
                          ("append", "/path1", "prepend", "/path2", "/path2:MyVar:/path1"),
                          ("append", "/path1", "unset", "", "/path1"),
                          ("prepend", "/path1", "define", "/path2", "/path1:/path2"),
                          ("prepend", "/path1", "append", "/path2", "/path1:MyVar:/path2"),
                          ("prepend", "/path1", "prepend", "/path2", "/path1:/path2:MyVar"),
                          ("prepend", "/path1", "unset", "", "/path1"),
                          ("unset", "", "define", "/path2", ""),
                          ("unset", "", "append", "/path2", ""),
                          ("unset", "", "prepend", "/path2", ""),
                          ("unset", "", "unset", "", ""),
                          ])
def test_compose_path_combinations(op1, v1, op2, v2, result):
    env = Environment()
    if op1 != "unset":
        getattr(env, op1+"_path")("MyVar", v1)
    else:
        env.unset("MyVar")
    env2 = Environment()
    if op2 != "unset":
        getattr(env2, op2+"_path")("MyVar", v2)
    else:
        env2.unset("MyVar")
    env.compose_env(env2)
    env = env.vars(ConanFileMock())
    assert env._values["MyVar"].get_str("{name}", None, pathsep=":") == result


def test_profile():
    myprofile = textwrap.dedent("""
        # define
        MyVar1 =MyValue1
        #  append
        MyVar2+=MyValue2
        #  multiple append works
        MyVar2+=MyValue2_2
        #  prepend
        MyVar3=+MyValue3
        # unset
        MyVar4=!
        # Empty
        MyVar5=

        # PATHS
        # define
        MyPath1=(path)/my/path1
        #  append
        MyPath2+=(path)/my/path2
        #  multiple append works
        MyPath2+=(path)/my/path2_2
        #  prepend
        MyPath3=+(path)/my/path3
        # unset
        MyPath4=!

        # PER-PACKAGE
        mypkg*:MyVar2=MyValue2
        """)

    profile_env = ProfileEnvironment.loads(myprofile)
    env = profile_env.get_profile_env("", is_consumer=True)
    env = env.vars(ConanFileMock())
    with environment_update({"MyVar1": "$MyVar1",
                             "MyVar2": "$MyVar2",
                             "MyVar3": "$MyVar3",
                             "MyVar4": "$MyVar4"}):
        assert env.get("MyVar1") == "MyValue1"
        assert env.get("MyVar2", "$MyVar2") == '$MyVar2 MyValue2 MyValue2_2'
        assert env.get("MyVar3", "$MyVar3") == 'MyValue3 $MyVar3'
        assert env.get("MyVar4") == ""
        assert env.get("MyVar5") == ''

        env = profile_env.get_profile_env(RecipeReference.loads("mypkg1/1.0"))
        env = env.vars(ConanFileMock())
        assert env.get("MyVar1") == "MyValue1"
        assert env.get("MyVar2", "$MyVar2") == 'MyValue2'


@pytest.fixture
def envvars():
    # Append and define operation over the same variable in Windows preserve order
    env = Environment()
    env.define("MyVar", "MyValueA")
    env.define("MYVAR", "MyValueB")
    env.define("MyVar1", "MyValue1A")
    env.append("MYVAR1", "MyValue1B")
    env.define("MyVar2", "MyNewValue2")

    conanfile = ConanFileMock()
    conanfile.settings_build = MockSettings({"os": "Windows", "arch": "x86_64"})
    env = env.vars(conanfile)
    env._subsystem = WINDOWS

    return env


def check_command_output(cmd, prevenv):
    result, _ = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                 env=prevenv, shell=True).communicate()
    out = result.decode()

    assert "MyVar=MyValueB!!" in out
    assert "MyVar=!!" in out
    assert "MyVar1=MyValue1A MyValue1B!!" in out
    assert "MyVar1=!!" in out
    assert "MyVar2=MyNewValue2!!" in out
    assert "MyVar2=OldValue2!!" in out


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
def test_windows_case_insensitive_bat(envvars):
    display = textwrap.dedent("""\
        @echo off
        echo MyVar=%MyVar%!!
        echo MyVar1=%MyVar1%!!
        echo MyVar2=%MyVar2%!!
        """)

    prevenv = {
        "MYVAR2": "OldValue2",
    }
    prevenv.update(dict(os.environ.copy()))  # Necessary from Python 3.12, for SYSTEMROOT var

    with chdir(temp_folder()):
        envvars.save_bat("test.bat")
        save("display.bat", display)
        cmd = "test.bat && display.bat && deactivate_test.bat && display.bat"
        check_command_output(cmd, prevenv)


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
@pytest.mark.parametrize("deactivation_mode", ["function", None],)
def test_windows_case_insensitive_ps1(envvars, deactivation_mode):
    display = textwrap.dedent("""\
        echo "MyVar=$env:MyVar!!"
        echo "MyVar1=$env:MyVar1!!"
        echo "MyVar2=$env:MyVar2!!"
        """)

    prevenv = {
        "MYVAR2": "OldValue2",
    }
    prevenv.update(dict(os.environ.copy()))

    with chdir(temp_folder()):
        envvars._deactivation_mode = deactivation_mode
        envvars.save_ps1("test.ps1")
        save("display.ps1", display)
        deactivate_cmd = "deactivate_test" if deactivation_mode else ".\\deactivate_test.ps1"
        cmd = f"powershell.exe .\\test.ps1 ; .\\display.ps1 ; {deactivate_cmd} ; .\\display.ps1"
        check_command_output(cmd, prevenv)


def test_dict_access():
    env = Environment()
    env.append("MyVar", "MyValue", separator="@")
    ret = env.vars(ConanFileMock()).items()
    assert dict(ret) == {"MyVar": "MyValue"}

    env = Environment()
    env.prepend("MyVar", "MyValue", separator="@")
    env_vars = env.vars(ConanFileMock())
    assert dict(env_vars.items()) == {"MyVar": "MyValue"}
    assert env_vars["MyVar"] == "MyValue"

    env2 = Environment()
    env2.define("MyVar", "MyValue2")
    env.compose_env(env2)
    env_vars = env.vars(ConanFileMock())
    assert dict(env_vars.items()) == {"MyVar": "MyValue@MyValue2"}

    with pytest.raises(KeyError):
        _ = env_vars["Missing"]

    # With previous values in the environment
    env = Environment()
    env.prepend("MyVar", "MyValue", separator="@")
    with environment_update({"MyVar": "PreviousValue"}):
        env_vars = env.vars(ConanFileMock())
        assert env_vars["MyVar"] == "MyValue@PreviousValue"

    env = Environment()
    env.append_path("MyVar", "MyValue")
    with environment_update({"MyVar": "PreviousValue"}):
        env_vars = env.vars(ConanFileMock())
        env_vars._subsystem = WINDOWS
        assert env_vars["MyVar"] == "PreviousValue;MyValue"
        with env_vars.apply():
            assert os.getenv("MyVar") == "PreviousValue;MyValue"

    assert list(env_vars.keys()) == ["MyVar"]
    assert dict(env_vars.items()) == {"MyVar": "MyValue"}


def test_env_win_bash():
    conanfile = ConanFileMock()
    conanfile.settings_build = MockSettings({"os": "Windows"})
    conanfile.win_bash = True
    conanfile.conf.define("tools.microsoft.bash:subsystem", "msys2")
    folder = temp_folder()
    conanfile.folders.generators = folder
    env = Environment()
    env.define("MyVar", "MyValue")
    env.define_path("MyPath", "c:/path/to/something")
    env.append("MyPath", "D:/Otherpath")
    env_vars = env.vars(conanfile)
    env_vars.save_script("foo")
    content = open(os.path.join(folder, "foo.sh")).read()
    assert 'MyVar="MyValue"' in content
    # Note the unit letter is lowercase
    assert 'MyPath="/c/path/to/something:/d/otherpath"' in content


def test_public_access():
    env = Environment()
    env.define("MyVar", "MyValue")
    env.append("MyVar", "MyValue2")
    env.define_path("MyPath", "c:/path/to/something")
    env.append_path("MyPath", "D:/Otherpath")
    env_vars = env.vars(ConanFileMock())
    env_vars._subsystem = WINDOWS
    for name, values in env_vars.items():
        if name == "MyVar":
            assert values == "MyValue MyValue2"
        if name == "MyPath":
            assert values == "c:/path/to/something;D:/Otherpath"

    env.remove("MyPath", "D:/Otherpath")
    assert env_vars.get("MyPath") == "c:/path/to/something"


class TestProfileEnvRoundTrip:

    def test_define(self):
        myprofile = textwrap.dedent("""
            # define
            MyVar1=MyValue1
            MyPath1 = (path)/my/path1
            """)

        env = ProfileEnvironment.loads(myprofile)
        text = env.dumps()
        assert text == textwrap.dedent("""\
            MyVar1=MyValue1
            MyPath1=(path)/my/path1
            """)

    def test_append(self):
        myprofile = textwrap.dedent("""
            # define
            MyVar1+=MyValue1
            MyPath1 +=(path)/my/path1
            """)

        env = ProfileEnvironment.loads(myprofile)
        text = env.dumps()
        assert text == textwrap.dedent("""\
            MyVar1+=MyValue1
            MyPath1+=(path)/my/path1
            """)

    def test_append_multiple(self):
        myprofile = textwrap.dedent("""
            # define
            MyVar1+=MyValue1
            MyVar1+=MyValue2
            MyPath1 +=(path)/my/path1
            MyPath1 +=(path)/my/path2
            """)

        env = ProfileEnvironment.loads(myprofile)
        text = env.dumps()
        assert text == textwrap.dedent("""\
            MyVar1+=MyValue1
            MyVar1+=MyValue2
            MyPath1+=(path)/my/path1
            MyPath1+=(path)/my/path2
            """)

    def test_prepend(self):
        myprofile = textwrap.dedent("""
            # define
            MyVar1=+MyValue1
            MyPath1=+(path)/my/path1
            """)

        env = ProfileEnvironment.loads(myprofile)
        text = env.dumps()
        assert text == textwrap.dedent("""\
            MyVar1=+MyValue1
            MyPath1=+(path)/my/path1
            """)

    def test_prepend_multiple(self):
        myprofile = textwrap.dedent("""
           # define
           MyVar1=+MyValue1
           MyVar1=+MyValue2
           MyPath1=+(path)/my/path1
           MyPath1=+(path)/my/path2
           """)

        env = ProfileEnvironment.loads(myprofile)
        text = env.dumps()
        assert text == textwrap.dedent("""\
           MyVar1=+MyValue1
           MyVar1=+MyValue2
           MyPath1=+(path)/my/path1
           MyPath1=+(path)/my/path2
           """)

    def test_combined(self):
        myprofile = textwrap.dedent("""
            MyVar1=+MyValue11
            MyVar1+=MyValue12
            MyPath1=+(path)/my/path11
            MyPath1+=(path)/my/path12
            """)

        env = ProfileEnvironment.loads(myprofile)
        text = env.dumps()
        assert text == textwrap.dedent("""\
            MyVar1=+MyValue11
            MyVar1+=MyValue12
            MyPath1=+(path)/my/path11
            MyPath1+=(path)/my/path12
            """)

    def test_combined_multiple(self):
        myprofile = textwrap.dedent("""
            MyVar1=+MyValue11
            MyVar1=+MyValue12
            MyVar1+=MyValue13
            MyVar1+=MyValue14
            MyPath1=+(path)/my/path11
            MyPath1=+(path)/my/path12
            MyPath1+=(path)/my/path13
            MyPath1+=(path)/my/path12
            """)

        env = ProfileEnvironment.loads(myprofile)
        text = env.dumps()
        assert text == textwrap.dedent("""\
            MyVar1=+MyValue11
            MyVar1=+MyValue12
            MyVar1+=MyValue13
            MyVar1+=MyValue14
            MyPath1=+(path)/my/path11
            MyPath1=+(path)/my/path12
            MyPath1+=(path)/my/path13
            MyPath1+=(path)/my/path12
            """)

    def test_combined_prepend_first(self):
        myprofile = textwrap.dedent("""
            MyVar1+=MyValue11
            MyVar1=+MyValue12
            MyPath1+=(path)/my/path11
            MyPath1=+(path)/my/path12
            """)

        env = ProfileEnvironment.loads(myprofile)
        text = env.dumps()
        # NOTE: This is reversed order compared to origin, prepend always first
        assert text == textwrap.dedent("""\
            MyVar1=+MyValue12
            MyVar1+=MyValue11
            MyPath1=+(path)/my/path12
            MyPath1+=(path)/my/path11
            """)

    def test_combined_prepend_first_multiple(self):
        myprofile = textwrap.dedent("""
            MyVar1+=MyValue11
            MyVar1=+MyValue12
            MyVar1+=MyValue13
            MyVar1=+MyValue14
            MyPath1+=(path)/my/path11
            MyPath1=+(path)/my/path12
            MyPath1+=(path)/my/path13
            MyPath1=+(path)/my/path14
            """)

        env = ProfileEnvironment.loads(myprofile)
        text = env.dumps()
        # NOTE: This is reversed order compared to origin, prepend always first
        assert text == textwrap.dedent("""\
            MyVar1=+MyValue12
            MyVar1=+MyValue14
            MyVar1+=MyValue11
            MyVar1+=MyValue13
            MyPath1=+(path)/my/path12
            MyPath1=+(path)/my/path14
            MyPath1+=(path)/my/path11
            MyPath1+=(path)/my/path13
            """)


def test_custom_placeholder():
    # https://github.com/conan-io/conan/issues/12427
    env = Environment()
    env.append_path("MyVar", "MyValue")
    env = env.vars(ConanFileMock())
    assert env.get("MyVar", variable_reference="$penv{{{name}}}") == \
           f"$penv{{MyVar}}:MyValue"
    items = {k: v for k, v in env.items(variable_reference="$penv{{{name}}}")}
    assert items == {"MyVar": f"$penv{{MyVar}}:MyValue"}


class TestProfileSeparators:

    def test_define(self):
        myprofile = "MyPath1 = (sep=@)/my/path1"
        env = ProfileEnvironment.loads(myprofile).get_profile_env(ref="")
        assert env.dumps() == "MyPath1=(sep=@)/my/path1"
        env.append("MyPath1", "/other/value")
        assert env.vars(ConanFileMock()).get("MyPath1") == "/my/path1@/other/value"

    def test_append(self):
        myprofile = "MyPath1 +=(sep=@)/my/path1"
        env = ProfileEnvironment.loads(myprofile).get_profile_env(ref="")
        assert env.dumps() == "MyPath1+=(sep=@)/my/path1"
        env.append("MyPath1", "/other/value")
        assert env.vars(ConanFileMock()).get("MyPath1") == "/my/path1@/other/value"

    def test_error(self):
        myprofile = "MyPath1 = (sep=@) (path)/my/path1"
        with pytest.raises(ConanException) as e:
            ProfileEnvironment.loads(myprofile)
        assert "Cannot use (sep) and (path) qualifiers simultaneously" in str(e.value)



import pytest

from conan.errors import ConanException
from conan.api.model import RecipeReference


def test_recipe_reference():
    r = RecipeReference.loads("pkg/0.1")
    assert r.name == "pkg"
    assert r.version == "0.1"
    assert r.user is None
    assert r.channel is None
    assert r.revision is None
    assert str(r) == "pkg/0.1"
    assert repr(r) == "pkg/0.1"

    r = RecipeReference.loads("pkg/0.1@")
    assert r.name == "pkg"
    assert r.version == "0.1"
    assert r.user is None
    assert r.channel is None
    assert r.revision is None
    assert str(r) == "pkg/0.1"
    assert repr(r) == "pkg/0.1"

    r = RecipeReference.loads("pkg/0.1@user")
    assert r.name == "pkg"
    assert r.version == "0.1"
    assert r.user == "user"
    assert r.channel is None
    assert r.revision is None
    assert str(r) == "pkg/0.1@user"
    assert repr(r) == "pkg/0.1@user"


def test_recipe_reference_revisions():
    r = RecipeReference.loads("pkg/0.1#r1")
    assert r.name == "pkg"
    assert r.version == "0.1"
    assert r.user is None
    assert r.channel is None
    assert r.revision == "r1"
    assert str(r) == "pkg/0.1"
    assert repr(r) == "pkg/0.1#r1"

    r = RecipeReference.loads("pkg/0.1@user#r1")
    assert r.name == "pkg"
    assert r.version == "0.1"
    assert r.user == "user"
    assert r.channel is None
    assert r.revision == "r1"
    assert str(r) == "pkg/0.1@user"
    assert repr(r) == "pkg/0.1@user#r1"


def test_recipe_reference_timestamp():
    r = RecipeReference.loads("pkg/0.1#r1%123")
    assert r.name == "pkg"
    assert r.version == "0.1"
    assert r.user is None
    assert r.channel is None
    assert r.revision == "r1"
    assert str(r) == "pkg/0.1"
    assert repr(r) == "pkg/0.1#r1%123.0"
    # TODO: Improve the time format
    assert r.repr_humantime() == "pkg/0.1#r1 (1970-01-01 00:02:03 UTC)"

    r = RecipeReference.loads("pkg/0.1@user#r1%123")
    assert r.name == "pkg"
    assert r.version == "0.1"
    assert r.user == "user"
    assert r.channel is None
    assert r.revision == "r1"
    assert str(r) == "pkg/0.1@user"
    assert repr(r) == "pkg/0.1@user#r1%123.0"
    assert r.repr_humantime() == "pkg/0.1@user#r1 (1970-01-01 00:02:03 UTC)"


def test_recipe_reference_compare():
    r1 = RecipeReference.loads("pkg/1.3#1")
    r2 = RecipeReference.loads("pkg/1.22#2")
    assert r1 < r2
    assert sorted([r2, r1]) == [r1, r2]


def test_error_pref():
    r1 = RecipeReference.loads("pkg/1.0#rrev1:pid#rrev2")
    with pytest.raises(ConanException) as exc:
        r1.validate_ref()
    assert "Invalid recipe reference 'pkg/1.0#rrev1:pid#rrev2' is a package reference" in str(exc)

    r1 = RecipeReference.loads("pkg/1.0:pid")
    with pytest.raises(ConanException) as exc:
        r1.validate_ref()
    assert "Invalid recipe reference 'pkg/1.0:pid' is a package reference" in str(exc)



import pytest

from conan.internal.model.cpp_info import CppInfo, _DIRS_VAR_NAMES, _FIELD_VAR_NAMES


def test_components_order():
    cppinfo = CppInfo()
    cppinfo.components["c1"].requires = ["c4", "OtherPackage::OtherComponent2"]
    cppinfo.components["c2"].requires = ["OtherPackage::OtherComponent"]
    cppinfo.components["c3"].requires = ["c2"]
    cppinfo.components["c4"].requires = ["c3"]
    sorted_c = list(cppinfo.get_sorted_components().keys())
    assert sorted_c == ["c2", "c3", "c4", "c1"]


def test_component_aggregation():
    cppinfo = CppInfo()

    cppinfo.includedirs = ["includedir"]
    cppinfo.libdirs = ["libdir"]
    cppinfo.srcdirs = ["srcdir"]
    cppinfo.bindirs = ["bindir"]
    cppinfo.builddirs = ["builddir"]
    cppinfo.frameworkdirs = ["frameworkdir"]
    cppinfo.set_property("foo", "bar")

    cppinfo.components["c2"].includedirs = ["includedir_c2"]
    cppinfo.components["c2"].libdirs = ["libdir_c2"]
    cppinfo.components["c2"].srcdirs = ["srcdir_c2"]
    cppinfo.components["c2"].bindirs = ["bindir_c2"]
    cppinfo.components["c2"].builddirs = ["builddir_c2"]
    cppinfo.components["c2"].frameworkdirs = ["frameworkdir_c2"]
    cppinfo.components["c2"].cxxflags = ["cxxflags_c2"]
    cppinfo.components["c2"].defines = ["defines_c2"]
    cppinfo.components["c2"].set_property("my_foo", ["bar", "bar2"])
    cppinfo.components["c2"].set_property("cmake_build_modules", ["build_module_c2",
                                                                  "build_module_c22"])

    cppinfo.components["c1"].requires = ["c2", "LIB_A::C1"]
    cppinfo.components["c1"].includedirs = ["includedir_c1"]
    cppinfo.components["c1"].libdirs = ["libdir_c1"]
    cppinfo.components["c1"].srcdirs = ["srcdir_c1"]
    cppinfo.components["c1"].bindirs = ["bindir_c1"]
    cppinfo.components["c1"].builddirs = ["builddir_c1"]
    cppinfo.components["c1"].frameworkdirs = ["frameworkdir_c1"]
    cppinfo.components["c1"].cxxflags = ["cxxflags_c1"]
    cppinfo.components["c1"].defines = ["defines_c1"]
    cppinfo.components["c1"].set_property("my_foo", "jander")
    cppinfo.components["c1"].set_property("my_foo2", "bar2")

    ret = cppinfo.aggregated_components()

    assert ret.get_property("foo") == "bar"
    assert ret.includedirs == ["includedir_c1", "includedir_c2"]
    assert ret.libdirs == ["libdir_c1", "libdir_c2"]
    assert ret.srcdirs == ["srcdir_c1", "srcdir_c2"]
    assert ret.bindirs == ["bindir_c1", "bindir_c2"]
    assert ret.builddirs == ["builddir_c1", "builddir_c2"]
    assert ret.frameworkdirs == ["frameworkdir_c1", "frameworkdir_c2"]
    assert ret.cxxflags == ["cxxflags_c1", "cxxflags_c2"]
    assert ret.defines == ["defines_c1", "defines_c2"]
    # The properties are not aggregated because we cannot generalize the meaning of a property
    # that belongs to a component, it could make sense to aggregate it or not, "cmake_target_name"
    # for example, cannot be aggregated. But "cmake_build_modules" is aggregated.
    assert ret.get_property("my_foo") is None
    assert ret.get_property("my_foo2") is None
    assert ret.get_property("cmake_build_modules") == None

    # If we change the internal graph the order is different
    cppinfo.components["c1"].requires = []
    cppinfo.components["c2"].requires = ["c1"]

    cppinfo._aggregated = None  # Dirty, just to force recomputation
    ret = cppinfo.aggregated_components()

    assert ret.includedirs == ["includedir_c2", "includedir_c1"]
    assert ret.libdirs == ["libdir_c2", "libdir_c1"]
    assert ret.srcdirs == ["srcdir_c2", "srcdir_c1"]
    assert ret.bindirs == ["bindir_c2", "bindir_c1"]
    assert ret.builddirs == ["builddir_c2", "builddir_c1"]
    assert ret.frameworkdirs == ["frameworkdir_c2", "frameworkdir_c1"]


def test_cpp_info_sysroot_merge():
    # If the value was already set is kept in the merge
    one = CppInfo()
    one.sysroot = "sys1"
    two = CppInfo()
    two.sysroot = "sys2"
    one.merge(two)
    assert one.sysroot == "sys1"

    # If the value was not set it is assigned
    one = CppInfo()
    two = CppInfo()
    two.sysroot = "sys2"
    one.merge(two)
    assert one.sysroot == "sys2"


@pytest.mark.parametrize("aggregate_first", [True, False])
def test_cpp_info_merge_aggregating_components_first(aggregate_first):
    cppinfo = CppInfo()
    for n in _DIRS_VAR_NAMES + _FIELD_VAR_NAMES:
        setattr(cppinfo.components["foo"], n, ["var_{}_1".format(n), "var_{}_2".format(n)])
        setattr(cppinfo.components["foo2"], n, ["var2_{}_1".format(n), "var2_{}_2".format(n)])

    cppinfo.components["foo"].requires = ["foo2"]  # Deterministic order

    other = CppInfo()
    for n in _DIRS_VAR_NAMES + _FIELD_VAR_NAMES:
        setattr(other.components["boo"], n, ["jar_{}_1".format(n), "jar_{}_2".format(n)])
        setattr(other.components["boo2"], n, ["jar2_{}_1".format(n), "jar2_{}_2".format(n)])

    other.components["boo"].requires = ["boo2"]  # Deterministic order

    if aggregate_first:
        cppinfo = cppinfo.aggregated_components()
        other = other.aggregated_components()

    cppinfo.merge(other)

    if aggregate_first:
        for n in _DIRS_VAR_NAMES + _FIELD_VAR_NAMES:
            assert getattr(cppinfo, n) == ["var_{}_1".format(n), "var_{}_2".format(n),
                                           "var2_{}_1".format(n), "var2_{}_2".format(n),
                                           "jar_{}_1".format(n), "jar_{}_2".format(n),
                                           "jar2_{}_1".format(n), "jar2_{}_2".format(n)]
    else:
        for n in _DIRS_VAR_NAMES + _FIELD_VAR_NAMES:
            assert getattr(cppinfo.components["foo"], n) == ["var_{}_1".format(n),
                                                             "var_{}_2".format(n)]
            assert getattr(cppinfo.components["foo2"], n) == ["var2_{}_1".format(n),
                                                              "var2_{}_2".format(n)]
            assert getattr(cppinfo.components["boo"], n) == ["jar_{}_1".format(n),
                                                             "jar_{}_2".format(n)]
            assert getattr(cppinfo.components["boo2"], n) == ["jar2_{}_1".format(n),
                                                              "jar2_{}_2".format(n)]
            assert getattr(cppinfo, n) == None



import os

from conan.internal.api.config.config_installer import tmp_config_install_folder
from conan.test.utils.test_files import temp_folder


class TestInstallFolder:

    def test_unique_install_folder(self):
        """ Validate if tmp_config_install_folder is removing old folder before creating a new one

        tmp_config_install_folder must create the same folder, but all items must be exclude when a
        new folder is created.
        """
        cache_folder = temp_folder()

        with tmp_config_install_folder(cache_folder) as tmp_folder_first:
            temp_file = os.path.join(tmp_folder_first, "foobar.txt")
            open(temp_file, "w+")
            with tmp_config_install_folder(cache_folder) as tmp_folder_second:
                assert tmp_folder_first == tmp_folder_second
                assert not os.path.exists(temp_file)



import textwrap

import pytest

from conan.tools.microsoft import unix_path, unix_path_package_info_legacy
from conan.internal.model.conf import ConfDefinition
from conan.test.utils.mocks import MockSettings, ConanFileMock

expected_results = [
    ("msys2", '/c/path/to/stuff'),
    ("msys", '/c/path/to/stuff'),
    ("cygwin", '/cygdrive/c/path/to/stuff'),
    ("wsl", '/mnt/c/path/to/stuff'),
]


@pytest.mark.parametrize("subsystem, expected_path", expected_results)
def test_unix_path(subsystem, expected_path):
    c = ConfDefinition()
    c.loads(textwrap.dedent("""\
        tools.microsoft.bash:subsystem={}
        tools.microsoft.bash:active=True
    """.format(subsystem)))

    settings = MockSettings({"os": "Windows"})
    conanfile = ConanFileMock()
    conanfile.conf = c.get_conanfile_conf(None)
    conanfile.settings = settings
    conanfile.settings_build = settings

    test_path = "c:/path/to/stuff"
    path = unix_path(conanfile, test_path)
    assert expected_path == path

    package_info_legacy_path = unix_path_package_info_legacy(conanfile, test_path, path_flavor=subsystem)
    assert package_info_legacy_path == test_path



import textwrap

import pytest

from conan.test.utils.tools import TestClient


@pytest.mark.tool("cmake")
def test_transitive_build_scripts():
    c = TestClient()
    scriptsa = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import copy
        class ScriptsA(ConanFile):
            name = "scriptsa"
            version = "0.1"
            package_type = "build-scripts"
            exports_sources = "*.cmake"
            def package(self):
                copy(self, "*.cmake", src=self.source_folder, dst=self.package_folder)
            def package_info(self):
                self.cpp_info.builddirs = ["."]
            """)
    scriptsa_cmake = textwrap.dedent("""
        function(myfunctionA)
            message(STATUS "MYFUNCTION CMAKE A: Hello world A!!!")
        endfunction()
    """)
    scriptsb = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import copy
        class ScriptsA(ConanFile):
            name = "scriptsb"
            version = "0.1"
            package_type = "build-scripts"
            exports_sources = "*.cmake"
            def requirements(self):
                self.requires("scriptsa/0.1", run=True, visible=True)
            def package(self):
                copy(self, "*.cmake", src=self.source_folder, dst=self.package_folder)
            def package_info(self):
                self.cpp_info.builddirs = ["."]
            """)
    scriptsb_cmake = textwrap.dedent("""
        find_package(scriptsa)
        function(myfunctionB)
            message(STATUS "MYFUNCTION CMAKE B: Hello world B!!!")
            myfunctionA()
        endfunction()
        """)
    app = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import CMake

        class App(ConanFile):
            package_type = "application"
            generators = "CMakeToolchain"
            settings = "os", "compiler", "arch", "build_type"

            def build_requirements(self):
                self.tool_requires("scriptsb/0.1")
            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()
        """)
    app_cmake = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(App LANGUAGES NONE)
        find_package(scriptsb)
        myfunctionB()
    """)
    c.save({"scriptsa/conanfile.py": scriptsa,
            "scriptsa/Findscriptsa.cmake": scriptsa_cmake,
            "scriptsb/conanfile.py": scriptsb,
            "scriptsb/Findscriptsb.cmake": scriptsb_cmake,
            "app/conanfile.py": app,
            "app/CMakeLists.txt": app_cmake})

    c.run("create scriptsa")
    c.run("create scriptsb")
    c.run("build app")

    assert "MYFUNCTION CMAKE B: Hello world B!!!" in c.out
    assert "MYFUNCTION CMAKE A: Hello world A!!!" in c.out


@pytest.mark.tool("cmake")
def test_reuse_macro_from_dep():
    """
    A simple cmake script can be added from a regular requires, without even a ``find_package()``
    https://github.com/conan-io/conan/issues/14013
    """
    c = TestClient()
    pkg = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import copy
        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"
            package_type = "static-library"
            exports_sources = "*.cmake"
            def package(self):
                copy(self, "*.cmake", src=self.source_folder, dst=self.package_folder)
            def package_info(self):
                self.cpp_info.builddirs = ["."]
            """)
    pkg_macros = textwrap.dedent("""
        function(pkg_macro)
            message(STATUS "PKG MACRO WORKING!!!")
        endfunction()
    """)
    app = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import CMake

        class App(ConanFile):
            package_type = "application"
            generators = "CMakeToolchain"
            settings = "os", "compiler", "arch", "build_type"

            def requirements(self):
                self.requires("pkg/0.1")

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()
        """)
    app_cmake = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(App LANGUAGES NONE)

        include(Macros)
        pkg_macro()
    """)
    c.save({"pkg/conanfile.py": pkg,
            "pkg/Macros.cmake": pkg_macros,
            "app/conanfile.py": app,
            "app/CMakeLists.txt": app_cmake})

    c.run("create pkg")
    c.run("build app")

    assert "PKG MACRO WORKING!!!" in c.out




conanfile_sources_v2 = '''from conan import ConanFile
from conan.tools.layout import basic_layout
from conan.tools.files import copy

class {{package_name}}Recipe(ConanFile):
    name = "{{name}}"
    version = "{{version}}"
    package_type = "header-library"

    # Optional metadata
    license = "<Put the package license here>"
    author = "<Put your name here> <And your email here>"
    url = "<Package recipe repository url here, for issues about the package>"
    description = "<Description of {{ name }} package here>"
    topics = ("<Put some tag here>", "<here>", "<and here>")

    # Sources are located in the same place as this recipe, copy them to the recipe
    exports_sources = "include/*"

    # Automatically manage the package ID clearing of settings and options
    implements = ["auto_header_only"]

    def layout(self):
        basic_layout(self)
    {% if requires is defined %}
    def requirements(self):
        {% for require in requires -%}
        self.requires("{{ require }}")
        {% endfor %}
    {%- endif %}

    def package(self):
        copy(self, "include/*", self.source_folder, self.package_folder)

    def package_info(self):
        self.cpp_info.bindirs = []
        self.cpp_info.libdirs = []

'''

source_h = """#pragma once

#include <iostream>
#include <vector>
#include <string>

{% if requires is defined -%}
{% for require in requires -%}
#include "{{ as_name(require) }}.h"
{% endfor %}
{%- endif %}

void {{package_name}}(){
    {% if requires is defined -%}
    {% for require in requires -%}
    {{ as_name(require).replace(".", "_") }}();
    {% endfor %}
    {%- endif %}

    std::cout << "{{package_name}}/{{version}} header only called" << std::endl;
}

"""

test_conanfile_v2 = """import os

from conan import ConanFile
from conan.tools.cmake import CMake, cmake_layout
from conan.tools.build import can_run


class {{package_name}}TestConan(ConanFile):
    settings = "os", "compiler", "build_type", "arch"
    generators = "CMakeDeps", "CMakeToolchain"

    def requirements(self):
        self.requires(self.tested_reference_str)

    def build(self):
        cmake = CMake(self)
        cmake.configure()
        cmake.build()

    def layout(self):
        cmake_layout(self)

    def test(self):
        if can_run(self):
            cmd = os.path.join(self.cpp.build.bindir, "example")
            self.run(cmd, env="conanrun")
"""

test_cmake_v2 = """cmake_minimum_required(VERSION 3.15)
project(PackageTest CXX)

find_package({{name}} CONFIG REQUIRED)

add_executable(example src/example.cpp)
target_link_libraries(example {{name}}::{{name}})
"""


test_main = """#include "{{name}}.h"
#include <vector>
#include <string>

int main() {
    {{package_name}}();
}
"""

header_only_lib_files = {"conanfile.py": conanfile_sources_v2,
                         "include/{{name}}.h": source_h,
                         "test_package/conanfile.py": test_conanfile_v2,
                         "test_package/src/example.cpp": test_main,
                         "test_package/CMakeLists.txt": test_cmake_v2}
